/*
	category_store.h



	2003   Plus Huang
 */

#ifndef __CATEGORY_STORE_H
#define __CATEGORY_STORE_H

#include "category.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef struct _CategoryStore CategoryStore;

#define CATEGORY_STORE_TAG    "category_store"

struct _CategoryStore {
	GStaticRecMutex     mutex;
	gint                n_thread;
	gint                n_thread_limit;

	GtkListStore*       list_store;
	GtkTreeStore*       tree_store;
};

enum CATEGORY_COLUMN_NAME {
	CATEGORY_COL_DATA,
	CATEGORY_COL_DOWNLOAD_STORE,
	CATEGORY_COL_DOWNLOAD_VIEW,
	CATEGORY_N_COL
};

void category_store_init (CategoryStore* cs);
void category_store_clear (CategoryStore* cs);

void category_store_append (CategoryStore* cs,
                            Category* cate);
void category_store_erase (CategoryStore* cs,
                           Category* cate);

Category* category_store_get_next (CategoryStore* cs,
                                   Category* category);

#define category_store_lock(cs)        g_static_rec_mutex_lock (&(cs)->mutex)
#define category_store_unlock(cs)      g_static_rec_mutex_unlock (&(cs)->mutex)

// thread manager for CategoryStore
void category_store_thread_ref (CategoryStore* cs);
void category_store_thread_unref (CategoryStore* cs);

void category_store_thread_activate_1 (CategoryStore* cs,
                                       Category* category);
void category_store_thread_activate_next (CategoryStore* cs,
                                          Category* category);

void category_store_thread_activate (CategoryStore* cs);
void category_store_thread_stop (CategoryStore* cs);
void category_store_thread_destroy (CategoryStore* ca);

#define category_store_thread_is_active(cs)   ((cs)->n_thread)

// xml
void category_store_write_xml (CategoryStore* store,
                               FILE* file, int level);

#ifdef __cplusplus
}
#endif

#endif  // End of __CATEGORY_STORE_H

