//  ConfParser
//
//  Copyright (C) 2003-2004 Plus Huang

#ifndef __CONF_PARSER_H
#define __CONF_PARSER_H

#include <glib.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef struct _ConfParser        ConfParser;
typedef struct _ConfParserEntry   ConfParserEntry;

// GMarkupParser func -----------------------------------------------
typedef void (*markup_parser_start_element_func) (GMarkupParseContext* gmpc,
                                                  const gchar*    element_name,
                                                  const gchar**   attr_names,
                                                  const gchar**   attr_values,
                                                  gpointer        data,
                                                  GError**        error);

typedef void (*markup_parser_end_element_func) (GMarkupParseContext*  gmpc,
                                                const gchar*      element_name,
                                                gpointer          data,
                                                GError**          error);

typedef void (*markup_parser_text_func) (GMarkupParseContext*  gmpc,
                                         const gchar*          text,
                                         gsize                 text_len,
                                         gpointer              data,
                                         GError**              error);

typedef void (*markup_parser_passthrough_func) (GMarkupParseContext*  gmpc,
                                                const gchar*  passthrough_text,
                                                gsize         text_len,  
                                                gpointer      data,
                                                GError**      error);

typedef void (*markup_parser_error_func) (GMarkupParseContext*  gmpc,
                                          GError*               error,
                                          gpointer              data);


// ConfParser func --------------------------------------------------
typedef void (*conf_parser_start_element_func) (ConfParser*       cp,
                                                const gchar*      element_name,
                                                const gchar**     attr_names,
                                                const gchar**     attr_values,
                                                gpointer          data,
                                                GError**          error);

typedef void (*conf_parser_end_element_func) (ConfParser*         cp,
                                              const gchar*        element_name,
                                              gpointer            data,
                                              GError**            error);

typedef void (*conf_parser_text_func) (ConfParser*          cp,
                                       const gchar*         text,
                                       gsize                text_len,
                                       gpointer             data,
                                       GError**             error);

typedef void (*conf_parser_passthrough_func) (ConfParser*     cp,
                                              const gchar*    passthrough_text,
                                              gsize           text_len,  
                                              gpointer        data,
                                              GError**        error);

typedef void (*conf_parser_error_func) (ConfParser*      cp,
                                        GError*          error,
                                        gpointer         data);

struct _ConfParserEntry {
	conf_parser_start_element_func  start_element;
	conf_parser_end_element_func    end_element;
	conf_parser_text_func           text;
	conf_parser_passthrough_func    passthrough;
	conf_parser_error_func          error;
};

struct _ConfParser {
	GMarkupParseContext* parse_context;
	GMarkupParser        markup_parser;

	GArray*     parser_stack;
	GPtrArray*  data_stack;

	ConfParserEntry*  parser;
	void*             data;
};

ConfParser* conf_parser_new ();
void conf_parser_destroy (ConfParser* cp);

void conf_parser_init (ConfParser* cp);
void conf_parser_finalize (ConfParser* cp);

gboolean conf_parser_read_file (ConfParser* cp,
                                const gchar* filename,
                                GError** error);

// push/pop both parser and data
void conf_parser_push (ConfParser* cp, ConfParserEntry* parser, void* data);
void conf_parser_pop (ConfParser* cp);
// push/pop parser only
void conf_parser_push_parser (ConfParser* cp, ConfParserEntry* parser);
void conf_parser_pop_parser (ConfParser* cp);
// push/pop data only
void conf_parser_push_data (ConfParser* cp, void* data);
void conf_parser_pop_data (ConfParser* cp);

#define conf_parser_get_element(cp)    g_markup_parse_context_get_element(cp->parse_context)

#ifdef __cplusplus
}
#endif

#endif  // End of __CONF_PARSER_H

