#include "download.h"

Download* download_new ()
{
	Download* dl = g_malloc (sizeof (Download));

	download_setting_init (&dl->setting);
	g_static_rec_mutex_init (&dl->mutex);

	dl->reference_count = 1;

	// info in downloading
	dl->completed = 0;
	dl->total = 0;
	dl->percent = 0;
	dl->speed = 0;
	dl->n_retries = 0;
	dl->message = NULL;

	dl->info_valid = FALSE;

	return dl;
}

Download* download_new_from_setting (DownloadSetting* ds)
{
	Download* dl = download_new ();

	download_setting_assign (&dl->setting, ds);

	return dl;
}

void download_assign (Download* dl, Download* dl_src)
{
	download_setting_assign (&dl->setting, &dl_src->setting);

	dl->completed = dl_src->completed;
	dl->n_retries = dl_src->n_retries;
	dl->percent   = dl_src->percent;
	dl->speed     = dl_src->speed;
	dl->total     = dl->total;

	dl->info_valid = dl_src->info_valid;

	download_set_message (dl, dl_src->message);
}

void download_ref (Download* dl)
{
	download_lock (dl);
	dl->reference_count++;
	download_unlock (dl);
}

void download_unref (Download* dl)
{
	gint  count;

	download_lock (dl);
	count = --dl->reference_count;
	download_unlock (dl);

	if (count > 0)
		return;

	g_static_rec_mutex_free (&dl->mutex);
	download_setting_clear (&dl->setting);
	g_free (dl->message);
	g_free (dl);
}


