//
//	download.h
//
//
//
//	2003   Plus Huang
//

#ifndef __DOWNLOAD_H
#define __DOWNLOAD_H

#include <glib.h>
#include "download_setting.h"

#ifdef __cplusplus
extern "C" {
#endif

//
//   ProxySetting
//   |
//   `--DownloadSetting
//      |
//      `--Download
//

#define DOWNLOAD(o)          ((Download*)(o))
#define DOWNLOAD_TAG         "download"

typedef struct _Download      Download;

struct _Download {
	DownloadSetting     setting;

	// private data
	GStaticRecMutex     mutex;
	gint                reference_count;

	// info in downloading
	gdouble completed;     // completed size
	gdouble total;         // total size
	gdouble percent;       // completed percent
	gdouble speed;         // download speed
	guint   n_retries;     // retry times
	gchar*  message;       // error message

	gboolean info_valid;   // info is valid in downloading.
};

Download* download_new ();
Download* download_new_from_setting (DownloadSetting* ds);

void download_assign (Download* dl, Download* dl_src);

void download_ref (Download* dl);
void download_unref (Download* dl);

#define download_lock(dl)        g_static_rec_mutex_lock (&(dl)->mutex)
#define download_unlock(dl)      g_static_rec_mutex_unlock (&(dl)->mutex)

#define download_set_message(dl, str)    \
            setting_set_string (&(dl)->message, str)

#ifdef __cplusplus
}
#endif

#endif  // End of __DOWNLOAD_H

