/*
	download_list_view.h


	2003  Plus Huang
 */

#include "download_list_view.h"
#include "setting_view_icons.h"
#include "urlgfe.h"
#include "dnd_data.h"

static gchar* dtoa_unit (double d, gchar* tail_str)
{
	const static gchar* unit_str[4] = {"", " KB", " MB", " GB"};
	guint  index;
	gint   precision = 0;

	for (index=0; index < 3; index++) {
		if (d > 1024.0) {
			d /= 1024.0;
			precision = 2;
		}
		else
			break;
	}

	return g_strdup_printf ("%.*f%s%s", precision, d, unit_str[index], tail_str);
}

// data set function for GtkCellRenderer
static void col_set_icon (GtkTreeViewColumn *tree_column,
                          GtkCellRenderer   *cell,
                          GtkTreeModel      *model,
                          GtkTreeIter       *iter,
                          gpointer           data)
{
	DownloadSetting* setting;

	gtk_tree_model_get (model, iter,
	                    DOWNLOAD_COL_DATA,
	                    &setting,
	                    -1);
	if (setting->state >= DOWNLOAD_N_STATE)
		setting->state = DOWNLOAD_STATE_ERROR;

	g_object_set ((GtkCellRenderer*)cell,
	              "pixbuf",
	              download_state_icons[setting->state],
	              NULL);
}

static void col_set_filename (GtkTreeViewColumn *tree_column,
                              GtkCellRenderer   *cell,
                              GtkTreeModel      *model,
                              GtkTreeIter       *iter,
                              gpointer           data)
{
	DownloadSetting* dd;

	gtk_tree_model_get (model, iter,
	                    DOWNLOAD_COL_DATA, &dd,
	                    -1);
	g_object_set ((GtkCellRenderer*)cell,
	              "text",
	              dd->filename,
	              NULL);
}

static void col_set_completed (GtkTreeViewColumn *tree_column,
                               GtkCellRenderer   *cell,
                               GtkTreeModel      *model,
                               GtkTreeIter       *iter,
                               gpointer           data)
{
	Download* dl_data;
	gpointer  thread;
	gchar*    string = NULL;

	gtk_tree_model_get (model, iter,
	                    DOWNLOAD_COL_DATA, &dl_data,
	                    DOWNLOAD_COL_THREAD, &thread,
	                    -1);

	if (thread && dl_data->info_valid)
		string = dtoa_unit (dl_data->completed, "");

	g_object_set ((GtkCellRenderer*)cell,
	              "text",
	              string,
	              NULL);
	g_free (string);
}

static void col_set_total (GtkTreeViewColumn *tree_column,
                           GtkCellRenderer   *cell,
                           GtkTreeModel      *model,
                           GtkTreeIter       *iter,
                           gpointer           data)
{
	Download* dl_data;
	gpointer  thread;
	gchar*    string = NULL;

	gtk_tree_model_get (model, iter,
	                    DOWNLOAD_COL_DATA, &dl_data,
	                    DOWNLOAD_COL_THREAD, &thread,
	                    -1);

	if (thread && dl_data->info_valid)
		string = dtoa_unit (dl_data->total, "");

	g_object_set ((GtkCellRenderer*)cell,
	              "text",
	              string,
	              NULL);
	g_free (string);
}

static void col_set_percent (GtkTreeViewColumn *tree_column,
                             GtkCellRenderer   *cell,
                             GtkTreeModel      *model,
                             GtkTreeIter       *iter,
                             gpointer           data)
{
	Download* dl_data;
	gpointer  thread;
	gchar*    string = NULL;

	gtk_tree_model_get (model, iter,
	                    DOWNLOAD_COL_DATA, &dl_data,
	                    DOWNLOAD_COL_THREAD, &thread,
	                    -1);

	if (thread && dl_data->info_valid)
		string = g_strdup_printf ("%.1f%%", dl_data->percent);

	g_object_set ((GtkCellRenderer*)cell,
	              "text",
	              string,
	              NULL);
	g_free (string);
}

static void col_set_speed (GtkTreeViewColumn *tree_column,
                           GtkCellRenderer   *cell,
                           GtkTreeModel      *model,
                           GtkTreeIter       *iter,
                           gpointer           data)
{
	Download* dl_data;
	gpointer  thread;
	gchar*    string = NULL;

	gtk_tree_model_get (model, iter,
	                    DOWNLOAD_COL_DATA, &dl_data,
	                    DOWNLOAD_COL_THREAD, &thread,
	                    -1);

	if (thread && dl_data->info_valid)
		string = dtoa_unit (dl_data->speed, "/s");

	g_object_set ((GtkCellRenderer*)cell,
	              "text",
	              string,
	              NULL);
	g_free (string);
}

static void col_set_retry (GtkTreeViewColumn *tree_column,
                           GtkCellRenderer   *cell,
                           GtkTreeModel      *model,
                           GtkTreeIter       *iter,
                           gpointer           data)
{
	Download* dl_data;
	gpointer  thread;
	gchar*    string = NULL;

	gtk_tree_model_get (model, iter,
	                    DOWNLOAD_COL_DATA, &dl_data,
	                    DOWNLOAD_COL_THREAD, &thread,
	                    -1);

	if (thread && dl_data->info_valid)
		string = g_strdup_printf ("%d", dl_data->n_retries);

	g_object_set ((GtkCellRenderer*)cell,
	              "text",
	              string,
	              NULL);
	g_free (string);
}

static void col_set_url (GtkTreeViewColumn *tree_column,
                         GtkCellRenderer   *cell,
                         GtkTreeModel      *model,
                         GtkTreeIter       *iter,
                         gpointer           data)
{
	DownloadSetting* dd;

	gtk_tree_model_get (model, iter,
	                    DOWNLOAD_COL_DATA, &dd,
	                    -1);
	g_object_set ((GtkCellRenderer*)cell,
	              "text",
	              dd->url,
	              NULL);
}


// Drag and drop - start ---------------------------------------
/*
static void on_drag_data_rcv (GtkWidget *widget,
                              GdkDragContext *dc,
                              guint x, guint y,
                              GtkSelectionData *sd,
                              guint info, guint t,
                              gpointer user_data)
{
	if (sd->target == gdk_atom_intern(DND_DOWNLOAD_DATA_TARGET, FALSE) &&
	    sd->data)
	{
		puts (DND_DOWNLOAD_DATA_TARGET);
	}
}
*/

static void on_drag_data_get (GtkWidget*        widget,
                              GdkDragContext*   dc,
                              GtkSelectionData* data,
                              guint info,
                              guint time,
                              DownloadListView* dlv)
{
	if (data->target == gdk_atom_intern(DND_DOWNLOAD_DATA_TARGET, FALSE)) {
		gtk_selection_data_set (data,
		                        gdk_atom_intern (DND_DOWNLOAD_DATA_TARGET, FALSE),
		                        8, /* bits */
		                        (void*)&dlv,
		                        sizeof (dlv));
	}
}

// Drag and drop - end ---------------------------------------


void download_list_view_init (DownloadListView* dlv,
                              DownloadStore* ds)
{
	GtkCellRenderer*   renderer;
	GtkTreeViewColumn* column;
	GtkTreeSelection*  selection;

	// for DnD
	GtkTargetEntry     gte = {
		DND_DOWNLOAD_DATA_TARGET,
		DND_DOWNLOAD_DATA_FLAG,
		DND_DOWNLOAD_DATA_INFO
	};

	// data set
	dlv->store = ds;

	// initial tree view.
	dlv->self = (GtkTreeView*)gtk_tree_view_new ();
	dlv->container = (GtkScrolledWindow*)gtk_scrolled_window_new (NULL, NULL);
	gtk_tree_view_set_model (dlv->self,
	                         GTK_TREE_MODEL(ds->list_store));
	gtk_tree_view_set_rules_hint (dlv->self, TRUE);

	// for GTK2 "floating" reference count.
	g_object_ref (G_OBJECT (dlv->container));
	gtk_object_sink (GTK_OBJECT (dlv->container));

	gtk_scrolled_window_set_policy (dlv->container,
	                                GTK_POLICY_AUTOMATIC,
	                                GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type (dlv->container,
	                                     GTK_SHADOW_IN);

	gtk_container_add (GTK_CONTAINER(dlv->container),
	                   GTK_WIDGET(dlv->self));

	// DnD support
	gtk_tree_view_enable_model_drag_source (dlv->self,
	                                        GDK_BUTTON2_MASK,
	                                        &gte, 1,
	                                        GDK_ACTION_COPY);
	g_signal_connect (G_OBJECT(dlv->self), "drag-data-get",
	                  G_CALLBACK(on_drag_data_get), dlv);
/*
	gtk_tree_view_enable_model_drag_dest (dlv->self,
	                                      gte, 1,
	                                      GDK_ACTION_COPY | GDK_ACTION_MOVE);
	g_signal_connect (G_OBJECT(dlv->self), "drag-data-received",
	                  G_CALLBACK(on_drag_data_rcv), dlv);
*/

	// set tree view selection
	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(dlv->self));
	gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE);

	// initial icons for tree view
	download_state_icons_render ((GtkWidget*)dlv->self);

	/* add columns to GtkTreeView */

	// column filename
	column = gtk_tree_view_column_new ();
	gtk_tree_view_column_set_title (column, _("Filename"));
	renderer = gtk_cell_renderer_pixbuf_new ();
	gtk_tree_view_column_pack_start (column, renderer, FALSE);
	gtk_tree_view_column_set_cell_data_func (column,
	                                         renderer,
	                                         col_set_icon,
	                                         NULL, NULL);
	renderer = gtk_cell_renderer_text_new ();
	gtk_tree_view_column_pack_start (column, renderer, TRUE);
	gtk_tree_view_column_set_cell_data_func (column,
	                                         renderer,
	                                         col_set_filename,
	                                         NULL, NULL);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_append_column (dlv->self, column);

	// column completed
	renderer = gtk_cell_renderer_text_new ();

	dlv->col_completed = gtk_tree_view_column_new ();
	gtk_tree_view_column_set_title (dlv->col_completed, _("Completed"));
	gtk_tree_view_column_pack_start (dlv->col_completed, renderer, TRUE);
	gtk_tree_view_column_set_cell_data_func (dlv->col_completed,
	                                         renderer,
	                                         col_set_completed,
	                                         NULL, NULL);
	gtk_tree_view_column_set_resizable (dlv->col_completed, TRUE);
	gtk_tree_view_append_column (dlv->self, dlv->col_completed);

	// column total
	dlv->col_total = gtk_tree_view_column_new ();
	gtk_tree_view_column_set_title (dlv->col_total, _("Total"));
	gtk_tree_view_column_pack_start (dlv->col_total, renderer, TRUE);
	gtk_tree_view_column_set_cell_data_func (dlv->col_total,
	                                         renderer,
	                                         col_set_total,
	                                         NULL, NULL);
	gtk_tree_view_column_set_resizable (dlv->col_total, TRUE);
	gtk_tree_view_column_set_min_width (dlv->col_total, 60);
	gtk_tree_view_append_column (dlv->self, dlv->col_total);

	// column percent
	dlv->col_percent = gtk_tree_view_column_new ();
	gtk_tree_view_column_set_title (dlv->col_percent, _("%"));
	gtk_tree_view_column_pack_start (dlv->col_percent, renderer, TRUE);
	gtk_tree_view_column_set_cell_data_func (dlv->col_percent,
	                                         renderer,
	                                         col_set_percent,
	                                         NULL, NULL);
	gtk_tree_view_column_set_resizable (dlv->col_percent, TRUE);
	gtk_tree_view_column_set_min_width (dlv->col_percent, 50);
	gtk_tree_view_append_column (dlv->self, dlv->col_percent);

	// columns speed
	dlv->col_speed = gtk_tree_view_column_new ();
	gtk_tree_view_column_set_title (dlv->col_speed, _("Speed"));
	gtk_tree_view_column_pack_start (dlv->col_speed, renderer, TRUE);
	gtk_tree_view_column_set_cell_data_func (dlv->col_speed,
	                                         renderer,
	                                         col_set_speed,
	                                         NULL, NULL);
	gtk_tree_view_column_set_resizable (dlv->col_speed, TRUE);
	gtk_tree_view_column_set_min_width (dlv->col_speed, 60);
	gtk_tree_view_append_column (dlv->self, dlv->col_speed);

	// column retries
	dlv->col_retry = gtk_tree_view_column_new ();
	gtk_tree_view_column_set_title (dlv->col_retry, _("Retry"));
	gtk_tree_view_column_pack_start (dlv->col_retry, renderer, TRUE);
	gtk_tree_view_column_set_cell_data_func (dlv->col_retry,
	                                         renderer,
	                                         col_set_retry,
	                                         NULL, NULL);
//	gtk_tree_view_column_set_sizing (dlv->col_retry, GTK_TREE_VIEW_COLUMN_FIXED);
	gtk_tree_view_column_set_resizable (dlv->col_retry, TRUE);
	gtk_tree_view_append_column (dlv->self, dlv->col_retry);

	// column url
	dlv->col_url = gtk_tree_view_column_new ();
	gtk_tree_view_column_set_title (dlv->col_url, _("URL"));
	renderer = gtk_cell_renderer_text_new ();
	gtk_tree_view_column_pack_start (dlv->col_url, renderer, TRUE);
	gtk_tree_view_column_set_cell_data_func (dlv->col_url,
	                                         renderer,
	                                         col_set_url,
	                                         NULL, NULL);
	gtk_tree_view_column_set_resizable (dlv->col_url, TRUE);
	gtk_tree_view_append_column (dlv->self, dlv->col_url);

	// show all widget
	gtk_widget_show_all ((GtkWidget*)dlv->container);
}

void download_list_view_clear (DownloadListView* dlv)
{
	gtk_widget_hide ((GtkWidget*)dlv->container);

	g_object_unref (dlv->container);
}

void download_list_view_hide_runtime_state (DownloadListView* dlv)
{
	gtk_tree_view_column_set_visible (dlv->col_completed, FALSE);
	gtk_tree_view_column_set_visible (dlv->col_percent, FALSE);
	gtk_tree_view_column_set_visible (dlv->col_retry, FALSE);
	gtk_tree_view_column_set_visible (dlv->col_speed, FALSE);
	gtk_tree_view_column_set_visible (dlv->col_total, FALSE);
}

GList* download_list_view_selected_list_copy (DownloadListView* dlv, gint* n_selected)
{
	GtkTreeSelection* selection;
	GtkTreeModel*     model;
	GtkTreeIter       iter;
	GList*            selected_list = NULL;
	GList*            node;
	Download*         dl_data;
	guint             n_iter;

	selection = gtk_tree_view_get_selection (dlv->self);
	download_store_lock (dlv->store);
	if (n_iter = gtk_tree_selection_count_selected_rows (selection)) {
		selected_list = gtk_tree_selection_get_selected_rows (selection, &model);
		for (node=selected_list; node; node=node->next) {
			gtk_tree_model_get_iter (model, &iter, node->data);
			gtk_tree_model_get (model, &iter, DOWNLOAD_COL_DATA, &dl_data, -1);
			gtk_tree_path_free (node->data);
			node->data = dl_data;
			download_ref (dl_data);
		}
	}
	download_store_unlock (dlv->store);

	if (n_selected)
		*n_selected = n_iter;

	return selected_list;
}

void  download_list_view_selected_list_free (GList* list)
{
	g_list_foreach (list, (GFunc)download_unref, NULL);
	g_list_free (list);
}

