#include <string.h>
#include <stdlib.h>
#include "utility.h"
#include "urlgfe.h"
#include "download_setting.h"


DownloadSetting* download_setting_init (DownloadSetting* dd)
{
	proxy_setting_init(&dd->proxy);

	dd->n_parts     = 0;
	dd->n_redirect  = 0;
	dd->n_retries   = 99;
	dd->retry_delay = 3;
	dd->multiple    = 0;  // FALSE
	dd->start_mode = DOWNLOAD_START_AUTO;
	dd->state      = DOWNLOAD_STATE_WAITING;
	dd->url       = NULL;
	dd->filename  = NULL;
	dd->directory = NULL;
	dd->referer   = NULL;

	return dd;
}

void download_setting_clear (DownloadSetting* dd)
{
	g_free (dd->url);
	g_free (dd->filename);
	g_free (dd->directory);
	g_free (dd->referer);
	dd->url       = NULL;
	dd->filename  = NULL;
	dd->directory = NULL;
	dd->referer   = NULL;

	proxy_setting_clear (&dd->proxy);
}

void download_setting_destroy (DownloadSetting* dd)
{
	download_setting_clear (dd);
	g_free (dd);
}

void download_setting_assign (DownloadSetting* dd, DownloadSetting* src)
{
	proxy_setting_assign (&dd->proxy, &src->proxy);

	dd->n_parts     = src->n_parts;
	dd->n_redirect  = src->n_redirect;
	dd->n_retries   = src->n_retries;
	dd->retry_delay = src->retry_delay;
//	dd->multiple    = src->multiple;
	dd->state       = src->state;
	download_setting_set_start_mode (dd, src->start_mode);
	download_setting_set_directory (dd, src->directory);
	download_setting_set_referer (dd, src->referer);
	if (dd->multiple==0 && src->multiple==0) {
		download_setting_set_filename (dd, src->filename);
		download_setting_set_url (dd, src->url);
	}
}

void download_setting_set_start_mode (DownloadSetting* dd, int mode)
{
	if (dd->state==DOWNLOAD_STATE_WAITING && mode==DOWNLOAD_START_MANUAL)
		dd->state = DOWNLOAD_STATE_PAUSE;
	else if (dd->state==DOWNLOAD_STATE_PAUSE && mode==DOWNLOAD_START_AUTO)
		dd->state = DOWNLOAD_STATE_WAITING;
	dd->start_mode = mode;
}

void download_setting_write_xml (DownloadSetting* setting,
                                 FILE* file, int level)
{
	gchar* url;
	gchar* filename;
	gchar* directory;
	gchar* referer;
	gint state = setting->state;

	if (state == DOWNLOAD_STATE_EXECUTING || state == DOWNLOAD_STATE_RETRY )
		state = DOWNLOAD_STATE_WAITING;

	url       = str_to_xml_str (STR_N2E (setting->url));
	filename  = str_to_xml_str (STR_N2E (setting->filename));
	directory = str_to_xml_str (STR_N2E (setting->directory));
	referer   = str_to_xml_str (STR_N2E (setting->referer));

	fprintf (file,
	         "%*c"                   // Start of format string
             "<" DOWNLOAD_SETTING_TAG " "
	         "parts=\"%d\" "
	         "redirect=\"%d\" "
			 "retries=\"%d\" "
			 "retry_delay=\"%d\" "
	         "multiple=\"%d\" "
	         "start_mode=\"%d\" "
	         "state=\"%d\">\n",      // End of format string
	         level, ' ',
	         setting->n_parts,
	         setting->n_redirect,
	         setting->n_retries,
	         setting->retry_delay,
	         setting->multiple,
	         setting->start_mode,
	         state
	         );
	proxy_setting_write_xml (PROXY_SETTING (setting), file, level+1);

	fprintf (file,
	         "%*c"
	         "<url>%s</url>\n"
	         "%*c"
	         "<directory>%s</directory>\n"
	         "%*c"
	         "<filename>%s</filename>\n"
	         "%*c"
	         "<referer>%s</referer>\n",
	         level+1, ' ',
	         url,
	         level+1, ' ',
	         directory,
	         level+1, ' ',
	         filename,
	         level+1, ' ',
	         referer
	         );

	fprintf (file, "%*c</" DOWNLOAD_SETTING_TAG ">\n", level, ' ');

	g_free (url);
	g_free (filename);
	g_free (directory);
	g_free (referer);
}

