#include <string.h>
#include "download_setting_view.h"
#include "urlgfe.h"
#include "utility.h"

//	signal handler
static void on_choose_directory(DownloadSettingView* view)
{
	GtkFileSelection* selector;

	selector = (GtkFileSelection*)gtk_file_selection_new (_("Choose a directory"));
	gtk_file_selection_complete (selector,
	                             gtk_entry_get_text (GTK_ENTRY(view->entry_dir)));
	gtk_widget_set_sensitive (selector->file_list, FALSE);
	gtk_widget_hide (selector->file_list);
	gtk_window_set_transient_for (GTK_WINDOW(selector),
	                              SETTING_VIEW(view)->parent);

	if (gtk_dialog_run (GTK_DIALOG(selector))==GTK_RESPONSE_OK ) {
		gtk_entry_set_text (GTK_ENTRY(view->entry_dir),
		                    gtk_file_selection_get_filename (selector));
	}
	gtk_widget_destroy (GTK_WIDGET(selector));
}

static gboolean on_key_release_event (DownloadSettingView* view)
{
	download_setting_view_url_changed (view);
	return FALSE;
}

static void on_paste_clipboard (GtkEntry* e, DownloadSettingView* view)
{
	download_setting_view_url_changed (view);
}

static void on_destroy (DownloadSettingView* view)
{
	g_string_free (view->last_filename, TRUE);
}

//	DownloadSettingView method
//  return : if URL is valid or MULTI_DOWNLOAD mode return TRUE.
gboolean download_setting_view_url_changed (DownloadSettingView* view)
{
	UrlPart up;

	url_part (&up, gtk_entry_get_text (GTK_ENTRY(view->entry_url)));

	if ( view->last_filename->str && up.name_beg==NULL ) {
		g_string_set_size (view->last_filename, 0);
		gtk_entry_set_text (GTK_ENTRY(view->entry_filename), "");
	}
	else if ( up.name_beg && strcmp (up.name_beg, view->last_filename->str)!=0 )
	{
		gtk_entry_set_text (GTK_ENTRY(view->entry_filename),
		                    up.name_beg);
		g_string_assign (view->last_filename,
		                 up.name_beg);
	}

	return (up.protocol_beg && up.address_beg) ? TRUE : FALSE;
}

void download_setting_view_init (DownloadSettingView* view)
{
	GtkWidget* hbox;
	GtkWidget* vbox;
	GtkWidget* label;
	GtkWidget* frame;
	GtkSizeGroup* size_group;

	// initial base class : ProxySettingView
	proxy_setting_view_init (&view->proxy_setting_view);
	proxy_setting_view_set_multiple (PROXY_SETTING_VIEW (view), TRUE);

	view->multiple = FALSE;
	view->last_filename = g_string_sized_new (40);

	/* create widgets */
	view->self = gtk_vbox_new (FALSE, 3);
	view->radio_auto = gtk_radio_button_new_with_mnemonic (NULL,
	                                                       _("Au_tomatic"));
	view->radio_manual = gtk_radio_button_new_with_mnemonic_from_widget (
	                             (GtkRadioButton*)view->radio_auto,
	                             _("_Manual"));
	size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);

	// pack URL label & entry
	hbox = gtk_hbox_new (FALSE, 3);
	view->entry_url = gtk_entry_new ();
	gtk_entry_set_width_chars (GTK_ENTRY(view->entry_url), 30);
	view->label_url = gtk_label_new_with_mnemonic (_("_URL :"));
	gtk_size_group_add_widget (size_group,
	                           view->label_url);
	gtk_label_set_mnemonic_widget (GTK_LABEL(view->label_url),
	                               view->entry_url);
	gtk_box_pack_start (GTK_BOX(hbox), view->label_url, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(hbox), view->entry_url, TRUE, TRUE, 0);
	gtk_box_pack_start (GTK_BOX(view->self), hbox, FALSE, FALSE, 0);

	// pack "Save to" & folder button
	hbox = gtk_hbox_new (FALSE, 3);
	view->button_folder = gtk_button_new ();
	gtk_container_add (GTK_CONTAINER (view->button_folder),
	    gtk_image_new_from_stock (GTK_STOCK_OPEN, GTK_ICON_SIZE_SMALL_TOOLBAR));
	gtk_button_set_relief (GTK_BUTTON (view->button_folder),
	                       GTK_RELIEF_NONE);
	view->combo_dir = gtk_combo_new ();
	view->entry_dir = GTK_COMBO(view->combo_dir)->entry;
	label = gtk_label_new_with_mnemonic (_("S_ave to :"));
	gtk_size_group_add_widget (size_group, label);
	gtk_label_set_mnemonic_widget(GTK_LABEL(label),
	                              view->combo_dir);
	gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(hbox), view->combo_dir, TRUE, TRUE, 0);
	gtk_box_pack_start (GTK_BOX(hbox), view->button_folder, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(view->self), hbox, FALSE, FALSE, 0);

	// pack "rename" label & entry
	hbox = gtk_hbox_new (FALSE, 3);
	view->entry_filename = gtk_entry_new ();
	view->label_filename = gtk_label_new_with_mnemonic (_("Re_name :"));
	gtk_size_group_add_widget (size_group, view->label_filename);
	gtk_label_set_mnemonic_widget (GTK_LABEL(view->label_filename),
	                               view->entry_filename);
	gtk_box_pack_start (GTK_BOX(hbox), view->label_filename, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(hbox), view->entry_filename, TRUE, TRUE, 0);
	gtk_box_pack_start (GTK_BOX(view->self), hbox, FALSE, FALSE, 0);

	// pack "Referer" label & entry
	hbox = gtk_hbox_new (FALSE, 3);
	view->entry_referer = gtk_entry_new ();
	view->label_referer = gtk_label_new_with_mnemonic (_("Referer :"));
	gtk_size_group_add_widget (size_group, view->label_referer);
	gtk_label_set_mnemonic_widget (GTK_LABEL(view->label_referer),
	                               view->entry_referer);
	gtk_box_pack_start (GTK_BOX(hbox), view->label_referer, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(hbox), view->entry_referer, TRUE, TRUE, 0);
	gtk_box_pack_start (GTK_BOX(view->self), hbox, FALSE, FALSE, 0);

	hbox = gtk_hbox_new (FALSE, 3);
	frame = gtk_frame_new (_("Start"));
	vbox = gtk_vbox_new (FALSE, 3);
	gtk_box_pack_start (GTK_BOX(vbox), view->radio_auto, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(vbox), view->radio_manual, FALSE, FALSE, 0);
	gtk_container_set_border_width (GTK_CONTAINER(vbox), 3);
	gtk_container_add (GTK_CONTAINER(frame), vbox);
	gtk_box_pack_start (GTK_BOX(hbox), frame, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(view->self), hbox, FALSE, FALSE, 0);
	vbox = gtk_vbox_new (FALSE, 3);
	gtk_container_set_border_width (GTK_CONTAINER(vbox), 3);
	frame = gtk_frame_new (_("Other"));
	gtk_container_add (GTK_CONTAINER(frame), vbox);
	gtk_box_pack_start (GTK_BOX(hbox), frame, FALSE, FALSE, 0);

	size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
	hbox = gtk_hbox_new (FALSE, 3);
	view->spin_retries = gtk_spin_button_new_with_range (0.0, 99.0, 1.0);
	label = gtk_label_new_with_mnemonic (_("Number of retr_ies :"));
	gtk_label_set_mnemonic_widget (GTK_LABEL(label),
	                               view->spin_retries);
	gtk_size_group_add_widget (size_group, label);
	gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(hbox), view->spin_retries, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(vbox), hbox, FALSE, FALSE, 0);

	hbox = gtk_hbox_new (FALSE, 3);
	view->spin_delay = gtk_spin_button_new_with_range (0.0, 600.0, 1.0);
	label = gtk_label_new_with_mnemonic (_("R_etry delay (second) :"));
	gtk_label_set_mnemonic_widget (GTK_LABEL(label), view->spin_delay);
	gtk_size_group_add_widget (size_group, label);
	gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(hbox), view->spin_delay, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(vbox), hbox, FALSE, FALSE, 0);

	hbox = gtk_hbox_new(FALSE, 3);
	view->spin_redirect = gtk_spin_button_new_with_range (0.0, 30.0, 1.0);
	label = gtk_label_new_with_mnemonic (_("_Redirection limit :"));
	gtk_label_set_mnemonic_widget (GTK_LABEL(label),
	                               view->spin_redirect);
	gtk_size_group_add_widget (size_group, label);
	gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(hbox), view->spin_redirect, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(vbox), hbox, FALSE, FALSE, 0);

	gtk_widget_show_all (GTK_WIDGET(view->self));

	/* add proxy setting view in a frame */
	frame = gtk_frame_new (_("Proxy Setting"));
	gtk_container_add (GTK_CONTAINER(frame),
	                   view->proxy_setting_view.self);
	gtk_container_set_border_width (GTK_CONTAINER(view->proxy_setting_view.self), 5);
	gtk_box_pack_start (GTK_BOX(view->self), frame, FALSE, FALSE, 0);
	gtk_widget_show (frame);

	/* change setting view container */
	SETTING_VIEW (view)->view_container = view->self;

	g_signal_connect_swapped (view->self, "destroy",
	                          G_CALLBACK(on_destroy), view);
	g_signal_connect_swapped (view->button_folder, "clicked",
	                          G_CALLBACK(on_choose_directory), view);
	g_signal_connect_swapped (view->entry_url, "key-release-event",
	                          G_CALLBACK(on_key_release_event), view);
	g_signal_connect_after (view->entry_url, "paste-clipboard",
	                        G_CALLBACK(on_paste_clipboard), view);
}

void download_setting_view_set (DownloadSettingView* view,
                                DownloadSetting* setting)
{
	if( setting==NULL )
		return;
	proxy_setting_view_set (&view->proxy_setting_view,
	                        &setting->proxy);

	gtk_entry_set_text (GTK_ENTRY(view->entry_dir),
	                    STR_N2E(setting->directory));
	gtk_spin_button_set_value (GTK_SPIN_BUTTON(view->spin_redirect),
	                           setting->n_redirect);
	gtk_spin_button_set_value (GTK_SPIN_BUTTON(view->spin_retries),
	                           setting->n_retries);
	gtk_spin_button_set_value (GTK_SPIN_BUTTON(view->spin_delay),
	                           setting->retry_delay);

	gtk_entry_set_text (GTK_ENTRY(view->entry_referer),
	                    STR_N2E(setting->referer));

	if( view->multiple==FALSE && setting->multiple==FALSE )
	{
		gtk_entry_set_text (GTK_ENTRY(view->entry_url),
		                    STR_N2E(setting->url));
		gtk_entry_set_text (GTK_ENTRY(view->entry_filename),
		                    STR_N2E(setting->filename));
		if (setting->filename==NULL)
			download_setting_view_url_changed (view);
	}

	switch (setting->start_mode) {
	case DOWNLOAD_START_AUTO:
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(view->radio_auto),
		                              TRUE);
		break;
	case DOWNLOAD_START_MANUAL:
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(view->radio_manual),
		                              TRUE);
		break;
	}
}

void download_setting_view_get (DownloadSettingView* view,
                                DownloadSetting* setting)
{
	if( setting==NULL )
		return;
	proxy_setting_view_get (&view->proxy_setting_view,
	                        &setting->proxy);

	download_setting_set_directory (setting,
			gtk_entry_get_text (GTK_ENTRY(view->entry_dir)) );

	setting->n_redirect = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(view->spin_redirect));
	setting->n_retries = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(view->spin_retries));
	setting->retry_delay = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(view->spin_delay));

	download_setting_set_referer (setting,
	                    gtk_entry_get_text (GTK_ENTRY(view->entry_referer)) );

	if ( view->multiple==FALSE ) {
		download_setting_set_filename (setting,
				gtk_entry_get_text (GTK_ENTRY(view->entry_filename)) );
		download_setting_set_url (setting,
				gtk_entry_get_text (GTK_ENTRY(view->entry_url)) );
	}

	if ( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(view->radio_auto)) )
		download_setting_set_start_mode (setting, DOWNLOAD_START_AUTO);
	else
		download_setting_set_start_mode (setting, DOWNLOAD_START_MANUAL);
}

gint download_setting_view_check_field (DownloadSettingView* view)
{
	const gchar *dir, *fname, *url;
	gboolean valid_url;

	dir   = gtk_entry_get_text (GTK_ENTRY(view->entry_dir));
	fname = gtk_entry_get_text (GTK_ENTRY(view->entry_filename));
	url   = gtk_entry_get_text (GTK_ENTRY(view->entry_dir));

	// complete filename from URL & check URL.
	if (*gtk_entry_get_text (GTK_ENTRY(view->entry_url))==0)
		valid_url = download_setting_view_url_changed (view);

	if ( view->multiple ) {
		valid_url = TRUE;
		if (*dir==0)
			return DOWNLOAD_SETTING_UNCOMPLETE;
	}
	else if ( *dir==0 || *fname==0 || *url==0 )
		return DOWNLOAD_SETTING_UNCOMPLETE_MULTIPlE;

	if (valid_url)
		return DOWNLOAD_SETTING_COMPLETE;

	return DOWNLOAD_SETTING_UNCOMPLETE_URL;
}

void download_setting_view_set_history (DownloadSettingView* dsv,
                                        GList* directory_history)
{
	if (directory_history) {
		gtk_combo_set_popdown_strings (GTK_COMBO (dsv->combo_dir),
		                               directory_history);
	}
}

void download_setting_view_set_multiple (DownloadSettingView *view,
                                         gboolean multiple)
{
	view->multiple = multiple;

	if ( multiple ) {
		gtk_widget_hide (view->entry_url);
		gtk_widget_hide (view->entry_filename);
		gtk_widget_hide (view->label_url);
		gtk_widget_hide (view->label_filename);
		gtk_widget_set_sensitive (view->entry_url, FALSE);
		gtk_widget_set_sensitive (view->entry_filename, FALSE);
		gtk_widget_set_sensitive (view->label_url, FALSE);
		gtk_widget_set_sensitive (view->label_filename, FALSE);
	}
	else {
		gtk_widget_show (view->entry_url);
		gtk_widget_show (view->entry_filename);
		gtk_widget_show (view->label_url);
		gtk_widget_show (view->label_filename);
		gtk_widget_set_sensitive (view->entry_url, TRUE);
		gtk_widget_set_sensitive (view->entry_filename, TRUE);
		gtk_widget_set_sensitive (view->label_url, TRUE);
		gtk_widget_set_sensitive (view->label_filename, TRUE);
	}
}

