/*
	download_thread.h


	2003  Plus Huang
 */

#ifndef __DOWNLOAD_THREAD_H
#define __DOWNLOAD_THREAD_H

#include "download_store.h"
#include "category_store.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef struct _DownloadThread DownloadThread;

struct _DownloadThread {
	GThread*           gthread;
	GStaticRecMutex    mutex;
	DownloadThread*    prev;
	DownloadThread*    next;
	guint              thread_state;

	// downloading data
	Download*          download_data;
	GString*           out_filename;
	gboolean           resumable;
	gdouble            timer_count;

	Category*          category;
	GtkTreeIter        iter_download;
	GTimer*            timer;
};

enum DOWNLOAD_THREAD_STATE {
	DOWNLOAD_THREAD_READY,
	DOWNLOAD_THREAD_ACTIVE,
	DOWNLOAD_THREAD_SKIP,        // skip current job
	DOWNLOAD_THREAD_STOP,        // do something before destory
	DOWNLOAD_THREAD_JOB_ERASED,  // download job erased by user
	DOWNLOAD_THREAD_DESTROY,     // destroying thread and freeing resource
};

DownloadThread* download_thread_new (Category* category,
                                     GtkTreeIter* iter_download);

// notify event to DownloadThread
void download_thread_activate (DownloadThread* dt);
void download_thread_skip (DownloadThread* dt);
void download_thread_stop (DownloadThread* dt);
void download_thread_job_erased (DownloadThread* dt);
void download_thread_destroy (DownloadThread* dt);

void download_thread_update_screen (DownloadThread* dt);

// call it before thread finish
void download_thread_free_resource (DownloadThread* dt);

// lock and unlock for DownloadThread
#define download_thread_lock(dt)    g_static_rec_mutex_lock (&(dt)->mutex)
#define download_thread_unlock(dt)  g_static_rec_mutex_unlock (&(dt)->mutex)

void download_thread_retry_delay (DownloadThread* dt);

gboolean download_thread_test_file (DownloadThread* dt);
gboolean download_thread_load_next_job (DownloadThread* dt);

gdouble download_thread_get_timer (DownloadThread* dt);

// display/update job data to screen.
void download_thread_job_redraw (DownloadThread* dt);

// set job data
#define download_thread_job_set_state(dt, s)    \
            DOWNLOAD_SETTING ((dt)->download_data)->state = (s)

#define download_thread_job_set_total(dt, size)        \
            (dt)->download_data->total = (size)

#define download_thread_job_set_completed(dt, size)        \
            (dt)->download_data->completed = (size)

#define download_thread_job_set_speed(dt, sp)        \
            (dt)->download_data->speed = (sp)

#define download_thread_job_set_percent(dt, p)        \
            (dt)->download_data->percent = (p)

#define download_thread_job_set_retry(dt, n)        \
            (dt)->download_data->n_retries = (n)

#define download_thread_job_set_message(dt, m)      \
            download_set_message ((dt)->download_data, m)

#define download_thread_job_set_info_valid(dt, b)        \
            (dt)->download_data->info_valid = (b)

#ifdef __cplusplus
}
#endif

#endif  /* End of __DOWNLOAD_THREAD_H */

