#include <string.h>
#include <stdlib.h>
#include "global_setting.h"
#include "urlgfe.h"
#include "utility.h"


GlobalSetting* global_setting_new ()
{
	GlobalSetting* gs = g_malloc (sizeof (GlobalSetting));

	global_setting_init (gs);

	return gs;
}

void global_setting_destroy (GlobalSetting* gs)
{
	global_setting_finalize (gs);
	g_free (gs);
}

void global_setting_init (GlobalSetting* gs)
{
	category_setting_init (&gs->category);

	gs->category.multiple = 1;  // TRUE
	gs->n_active_category = 2;
	gs->header_category_visible = 1;

	gs->directory_list = NULL;
	gs->n_directory = 0;

	gs->extention_list = NULL;
	gs->extention_data = NULL;
	gs->extention_size = 0;
	gs->monitoring_clipboard = TRUE;
	g_static_rec_mutex_init (&gs->mutex);

	global_setting_set_extention (gs, "ZIP;BIN;GZ;Z;TAR;ARJ;LZH;MP3;RAR;RPM;DEB");
}

void global_setting_finalize (GlobalSetting* gs)
{
	category_setting_clear (&gs->category);
}

void global_setting_assign (GlobalSetting* gs, GlobalSetting* src)
{
	category_setting_assign (&gs->category, &src->category);

	gs->n_active_category = src->n_active_category;
	gs->monitoring_clipboard = src->monitoring_clipboard;
}

void   global_setting_add_directory (GlobalSetting* gs,
                                     const gchar* directory)
{
	GList* node;
	gchar* string;

	if (directory==NULL || *directory==0)
		return;

	node = g_list_find_custom (gs->directory_list,
	                           directory,
	                           (GCompareFunc)strcmp);

	if (node) {
		string = node->data;
		gs->directory_list = g_list_delete_link (gs->directory_list, node);
		gs->n_directory--;
	}
	else {
		if (gs->n_directory > 8) {
			if (node = g_list_last (gs->directory_list))
				g_free (node->data);
			gs->directory_list = g_list_delete_link (gs->directory_list, node);
			gs->n_directory--;
		}
		string = g_strdup (directory);
	}

	gs->directory_list = g_list_prepend (gs->directory_list, string);
	gs->n_directory++;
}

GList* global_setting_get_directory_list (GlobalSetting* gs)
{
	return gs->directory_list;
}

void global_setting_set_extention (GlobalSetting* gs, const gchar* ext)
{
	gboolean is_beg = TRUE;
	gchar*   ext_str;
	gchar*   ext_end;

	global_setting_lock (gs);

	// free data
	g_list_free (gs->extention_list);
	gs->extention_list = NULL;
	g_free (gs->extention_data);

	gs->extention_data = g_strdup (ext);
	gs->extention_size = strlen (ext);

	ext_str = gs->extention_data;
	ext_end = ext_str + gs->extention_size;
	while (ext_str < ext_end) {
		if (is_beg) {
			gs->extention_list = g_list_append (gs->extention_list, ext_str);
			is_beg = FALSE;
		}
		if (*ext_str == ';') {
			*ext_str = 0;
			is_beg = TRUE;
		}
		ext_str++;
	}

	global_setting_unlock (gs);
}

gchar* global_setting_get_extention (GlobalSetting* gs)
{
	gchar* ext_str;
	gchar* ext_cur;
	gchar* ext_end;
	gint   len;

	global_setting_lock (gs);

	// extension
	len = gs->extention_size;
	ext_str = g_memdup (gs->extention_data, len+1);
	ext_end = ext_str + len;

	global_setting_unlock (gs);

	for (ext_cur=ext_str; ext_cur < ext_end; ++ext_cur) {
		if (*ext_cur == 0)
			*ext_cur = ';';
	}

	return ext_str;
}

gboolean global_setting_find_extention (GlobalSetting* gs, const gchar* ext)
{
	GList*   node;
	gboolean return_val = FALSE;

	global_setting_lock (gs);

	for (node=gs->extention_list; node; node=node->next) {
		if (g_ascii_strcasecmp (node->data, ext)==0) {
			return_val = TRUE;
			break;
		}
	}

	global_setting_unlock (gs);

	return return_val;
}

void global_setting_write_xml (GlobalSetting* setting,
                               FILE* file, int level)
{
	GList* list;
	gchar* ext_str;
	gchar* dest_str;

	if (level)
		fprintf (file, "%*c", level, ' ');

	fprintf (file,
	         "<" GLOBAL_SETTING_TAG " "       // Start of format string
	         "n_active_category=\"%d\" "
	         "header_category_visible=\"%d\" "
	         "monitoring_clipboard=\"%d\">"
	         "\n",                            // End of format string
	         setting->n_active_category,
	         setting->header_category_visible,
	         setting->monitoring_clipboard
	         );
	category_setting_write_xml (CATEGORY_SETTING (setting), file, level+1);

	// write directory
	list = g_list_last (setting->directory_list);
	for ( ; list; list=list->prev) {
		fprintf (file,
		         "%*c"
		         "<directory>%s</directory>\n",
				 level, ' ',
		         list->data);
	}

	// extension
	ext_str = global_setting_get_extention (setting);
	dest_str = str_to_xml_str ((const gchar*)ext_str);
	g_free (ext_str);
	fprintf (file, "%*c" "<extention>%s</extention>\n",
			 level, ' ', dest_str);
	g_free (dest_str);

	if (level)
		fprintf (file, "%*c", level, ' ');

	fputs ("</" GLOBAL_SETTING_TAG ">\n", file);
}

