#include <stdio.h>
#include <string.h>
#include "html_import.h"
#include "selection_store.h"
#include "utility.h"

/*
static void html_import_start_element (GMarkupParseContext* context,
                                       const gchar*         element_name,
                                       const gchar**        attr_names,
                                       const gchar**        attr_values,
                                       HtmlImport*          hi,
                                       GError**             error)
*/
void html_import_start_element (HtmlImport*   hi,
                                const gchar*  element_name,
                                const gchar** attr_names,
                                const gchar** attr_values)
{
	GList* node;
	gint   index;
	HtmlImportElement* hie = NULL;

	// check tag <base href="some.base.address">
	if (g_ascii_strcasecmp (element_name, "base")==0) {
		for (index=0; attr_names[index]; index++) {
			if (g_ascii_strcasecmp (attr_names[index], "href")==0 ) {
				g_free (hi->base_href);
				hi->base_href = g_strdup (attr_values[index]);
			}
		}
	}

	// find tag with element_list
	for (node=hi->element_list; node; node=node->next)
	{
		hie = node->data;
		if (g_ascii_strcasecmp (element_name, hie->element_name)==0)
		{
			hie = node->data;
			// check attr_name
			for (index=0; attr_names[index]; index++)
			{
				if (g_ascii_strcasecmp (attr_names[index], hie->attr_name)==0)
				{
					html_import_element_add_url (hie, attr_values[index]);
					break;
				}
			}
			break;
		}
	}
}

HtmlImport* html_import_new()
{
	HtmlImport* hi = g_malloc (sizeof (HtmlImport));

	hi->base_href = NULL;

	hi->name_array = g_ptr_array_sized_new (16);
	hi->value_array = g_ptr_array_sized_new (16);

	hi->element_list = NULL;
	hi->gstring = g_string_sized_new (80);

	return hi;
}

void html_import_destroy (HtmlImport* hi)
{
	GList* node;

	g_free (hi->base_href);

	g_ptr_array_free (hi->name_array, TRUE);
	g_ptr_array_free (hi->value_array, TRUE);

	for (node=hi->element_list; node; node=node->next)
		html_import_element_destroy (node->data);

	g_list_free (hi->element_list);
	g_string_free (hi->gstring, TRUE);
	g_free (hi);
}

void html_import_add_grabbed_element (HtmlImport* html_import,
                                      gchar* element_name,
                                      gchar* attr_name,
                                      gchar* label_name)
{
	HtmlImportElement* hie;

	hie = html_import_element_new (element_name, attr_name, label_name);

	html_import->element_list = g_list_append (html_import->element_list,
	                                           hie);
}

HtmlImportElement* html_import_get_element (HtmlImport* html_import,
                                            gint index)
{
	GList* node = g_list_nth (html_import->element_list, index);

	return node->data;
}

void html_import_parse_tag (HtmlImport* hi, gchar* tag_str)
{
	gchar*   current      = tag_str;
	gchar*   element_name = NULL;
	gchar*   attr_name    = NULL;
	gchar*   attr_value   = NULL;
	gint     inside_level;
	gchar    inside_chr;

	g_ptr_array_set_size (hi->name_array, 0);
	g_ptr_array_set_size (hi->value_array, 0);

	// element
	while (*current) {
		if (*current == ' ') {
			*current++ = 0;
			element_name = tag_str;
			break;
		}
		current++;
	}

	if (element_name==NULL || *element_name==0)
		return;

	while (*current) {
		// skip space
		while (*current == ' ')
			current++;

		// attribute name
		attr_name  = current;
		while (*current) {
			if (*current == '=') {
				*current++ = 0;
				break;
			}
			else if (*current == ' ') {
				attr_name = NULL;
				break;
			}
			current++;
		}

		if (attr_name==NULL)
			continue;

		// attribute value
		attr_value = current;
		inside_level = 0;
		inside_chr   = 0;
		while (*current) {
			if (*current == '"' || *current == '\'') {
				if (inside_chr != *current) {
					if (inside_level==0)
						attr_value = current+1;
					inside_chr = *current;
					inside_level++;
				}
				else if (inside_chr == *current) {
					inside_chr = (inside_chr=='"') ? '\'' : '"';
					inside_level--;
				}

				if (current[1] == 0 || current[1] == ' ' || inside_level==0) {
					*current = 0;
					break;
				}
			}
			else if (*current == ' ' && inside_level == 0) {
				*current = 0;
				break;
			}
			current++;
		}
		// add attribute
		g_ptr_array_add (hi->name_array, attr_name);
		g_ptr_array_add (hi->value_array, attr_value);
	}
	g_ptr_array_add (hi->name_array, NULL);
	g_ptr_array_add (hi->value_array, NULL);

	// call
	html_import_start_element (hi, element_name,
	                           (const gchar**)hi->name_array->pdata,
	                           (const gchar**)hi->value_array->pdata);

}

gboolean html_import_read_file (HtmlImport* hi, gchar* filename)
{
	FILE*    file;
	guint8  *buffer, *data_end, *current;

	guint    read_size;
	gboolean inside_tag = FALSE;
	GString* gstring;

	file = fopen (filename, "r");

	if (file==NULL)
		return FALSE;

	buffer = g_malloc (4096);
	gstring = g_string_sized_new (4096);

	do {
		read_size = fread (buffer, 1, 4096, file);
		data_end = buffer + read_size;

		for (current=buffer; current!=data_end; current++) {
			switch (*current) {
			case '<':
				inside_tag = TRUE;
				g_string_truncate (gstring, 0);
				break;
			case '>':
				if (inside_tag)
					html_import_parse_tag (hi, gstring->str);
				inside_tag = FALSE;
				g_string_truncate (gstring, 0);
				break;
			case '\r':
			case '\n':
				// skip
				break;
			default:
				if (inside_tag)
					g_string_append_c (gstring, *current);
			}
			// check <tag> length
			if( gstring->len >= 4096 ) {
				inside_tag=FALSE;
				g_string_truncate(gstring, 0);
			}
		}
	} while (read_size);

	g_string_free (gstring, TRUE);
	g_free (buffer);

	fclose (file);

	return TRUE;
}

/*
gboolean html_import_read_file (HtmlImport* hi,
                                gchar* filename)
{
	GMarkupParseContext* parser_context;
	GMarkupParser        parser;

	FILE*    file;
	guint    size;
	guint8*  buffer;
	GError*  error = NULL;
	gboolean parse_ok = TRUE;

	parser.start_element = html_import_start_element;
	parser.end_element   = NULL;
	parser.text          = NULL;
	parser.passthrough   = NULL;
	parser.error         = NULL;
	parser_context = g_markup_parse_context_new (&parser, 0,
	                                             hi, NULL);

	file = fopen (filename, "r");
	if (file==NULL)
		return FALSE;

	buffer = g_malloc (4096);

	while ( (size=fread (buffer, 1, 4096, file)) && parse_ok) {
		parse_ok = g_markup_parse_context_parse (parser_context,
		                                         buffer, size, &error);
	}

	if (error)
		g_error_free (error);

	g_markup_parse_context_end_parse (parser_context, NULL);
	g_markup_parse_context_free (parser_context);

	g_free (buffer);
	fclose (file);

	return parse_ok;
}
*/

gboolean html_import_get_url_first (HtmlImport*     hi,
                                    HtmlImportIter* iter,
                                    const gchar**   string)
{
	iter->first = TRUE;
	iter->element_list = hi->element_list;

	return html_import_get_url_next (hi, iter, string);
}

gboolean html_import_get_url_next (HtmlImport*     hi,
                                   HtmlImportIter* iter,
                                   const gchar**   string)
{
	HtmlImportElement* hie;
	gboolean    valid;
	gboolean    selected;
	GString*    gstring = hi->gstring;
	UrlPart     urlp;

	for (; iter->element_list; iter->element_list=iter->element_list->next) {
		hie = iter->element_list->data;

		// check if first time search in this HtmlImportElement
		if (iter->first) {
			iter->first = FALSE;
			valid = gtk_tree_model_get_iter_first (GTK_TREE_MODEL (hie->url_store),
			                                       &iter->tree_iter);
		}
		else {
			valid = gtk_tree_model_iter_next (GTK_TREE_MODEL (hie->url_store),
			                                  &iter->tree_iter);
		}

		// search selected URL from HtmlImportElement
		while (valid) {
			gtk_tree_model_get (GTK_TREE_MODEL (hie->url_store),
			                    &iter->tree_iter,
			                    SELECTION_STORE_SELECTED, &selected,
			                    SELECTION_STORE_STRING,   string,
			                    -1);
			if (selected) {
				url_part (&urlp, *string);
				g_string_truncate (gstring, 0);

				if (urlp.protocol_beg==NULL) {
					url_part (&urlp, hi->base_href);
					g_string_append_len (gstring, urlp.protocol_beg,
					                     urlp.folder_end - urlp.protocol_beg);
				}
				g_string_append (gstring, *string);
				*string = gstring->str;
				return TRUE;
			}

			valid = gtk_tree_model_iter_next (GTK_TREE_MODEL (hie->url_store),
			                                  &iter->tree_iter);
		}
		iter->first = TRUE;
	}
	return FALSE;
}

// =================================================================

void html_import_class_init (HtmlImportClass* hic)
{
	hic->hash_table = g_hash_table_new_full (g_str_hash, g_str_equal,
	                                         (GDestroyNotify)g_free,
	                                         NULL);
	hic->index_array = g_array_sized_new (FALSE, FALSE, sizeof(gint), 100);
	hic->list_store = selection_store_new ();
}

void html_import_class_clear (HtmlImportClass* hic)
{
	g_hash_table_destroy (hic->hash_table);
	g_array_free (hic->index_array, TRUE);
	g_object_unref (hic->list_store);
}

void html_import_class_add (HtmlImportClass* hic,
                            const gchar* class_name)
{
	gint index;
	gpointer key, val;
	GtkTreeIter iter;

	if (class_name==NULL || *class_name==0)
		class_name="(null)";

	if( g_hash_table_lookup_extended (hic->hash_table,
	                                  class_name,
	                                  &key, &val) )
	{
		index = GPOINTER_TO_INT(val);
	}
	else {
		printf ("--- class:%s\n", class_name);
		gtk_list_store_append (hic->list_store, &iter);
		gtk_list_store_set (hic->list_store, &iter,
		                    SELECTION_STORE_STRING, class_name,
		                    -1);
		index = -1;
	}
	g_hash_table_insert (hic->hash_table,
	                     (gpointer)g_strdup(class_name),
	                     GINT_TO_POINTER(hic->index_array->len));
	g_array_append_val (hic->index_array, index);
}

void html_import_class_apply_selection_to_element (HtmlImportClass*   hic,
                                                   HtmlImportElement* hie)
{
	gboolean valid;
	gboolean selected;
	const gchar* string;
	GtkTreeIter iter;
	GtkTreeIter iter_list;
	gpointer key, val;
	gint index;

	valid = gtk_tree_model_get_iter_first (GTK_TREE_MODEL (hic->list_store),
	                                       &iter);
	while(valid) {
		gtk_tree_model_get (GTK_TREE_MODEL(hic->list_store), &iter,
		                    SELECTION_STORE_SELECTED, &selected,
		                    SELECTION_STORE_STRING,   &string,
		                    -1);
		if( selected ) {
			g_hash_table_lookup_extended (hic->hash_table, string,
			                              &key, &val);
			index = GPOINTER_TO_INT(val);
			while( index != -1 ) {
				gint count;
				gboolean selected;
				gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (hie->url_store),
				                               &iter_list,
				                               NULL, index);
				gtk_tree_model_get (GTK_TREE_MODEL (hie->url_store),
				                    &iter_list,
				                    SELECTION_STORE_COUNT, &count,
				                    -1);

				selected = (count) ? TRUE : FALSE;
				count++;

				gtk_list_store_set (hie->url_store, &iter_list,
				                    SELECTION_STORE_SELECTED, selected,
				                    SELECTION_STORE_COUNT,    count,
				                    -1);

				index = g_array_index (hic->index_array, gint, index);
			}
		}
		valid = gtk_tree_model_iter_next (GTK_TREE_MODEL (hic->list_store),
		                                  &iter);
	}
}

// =================================================================

HtmlImportElement* html_import_element_new (gchar* element_name,
                                            gchar* attr_name,
                                            gchar* label_name)
{
	HtmlImportElement* he = g_malloc (sizeof (HtmlImportElement));

	he->element_name = g_strdup (element_name);
	he->attr_name    = g_strdup (attr_name);
	he->label_name   = g_strdup (label_name);

	he->url_store = selection_store_new ();

	html_import_class_init (&he->extension);
	html_import_class_init (&he->address);

	return he;
}

void html_import_element_destroy (HtmlImportElement* he)
{
	g_free (he->element_name);
	g_free (he->attr_name);
	g_free (he->label_name);

	g_object_unref (he->url_store);

	html_import_class_clear (&he->extension);
	html_import_class_clear (&he->address);

	g_free (he);
}

void html_import_element_add_url (HtmlImportElement* he,
                                  const gchar* url)
{
	GtkTreeIter iter;
	gchar*  url_str;
	gchar*  ext = NULL;
	gchar*  addr = NULL;
	UrlPart urlp;

	url_str = g_strdup (url);
	url_part (&urlp, url_str);

	if (urlp.ext_end)
		ext = urlp.ext_beg;
	if (urlp.address_end) {
		*urlp.address_end = 0;
		addr = urlp.address_beg;
	}

	html_import_class_add (&he->extension, ext);
	html_import_class_add (&he->address, addr);

	gtk_list_store_append (he->url_store, &iter);
	gtk_list_store_set (he->url_store, &iter,
	                    SELECTION_STORE_STRING, url,
	                    -1);

	g_free (url_str);
}

void html_import_element_apply_selection (HtmlImportElement* hie)
{
	html_import_class_apply_selection_to_element (&hie->address, hie);
	html_import_class_apply_selection_to_element (&hie->extension, hie);
}

