#ifndef __HTML_IMPORT_H
#define __HTML_IMPORT_H

#include <gtk/gtk.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef struct _HtmlImport        HtmlImport;
typedef struct _HtmlImportIter    HtmlImportIter;
typedef struct _HtmlImportClass   HtmlImportClass;
typedef struct _HtmlImportElement HtmlImportElement;

// =================================================================

struct _HtmlImport {
	gchar*      base_href;    // for <base href="some.address">

	GPtrArray*  name_array;
	GPtrArray*  value_array;

	GList*      element_list;
	GString*    gstring;
};

struct _HtmlImportIter {
	GList*      element_list;
	GtkTreeIter tree_iter;
	gboolean    first;
};

HtmlImport* html_import_new();
void html_import_destroy (HtmlImport* html_import);

void html_import_add_grabbed_element (HtmlImport* html_import,
                                      gchar* element_name,
                                      gchar* attr_name,
                                      gchar* label_name);

HtmlImportElement* html_import_get_element (HtmlImport* html_import,
                                            gint index);

gboolean html_import_read_file (HtmlImport* hi, gchar* filename);

gboolean html_import_get_url_first (HtmlImport*     hi,
                                    HtmlImportIter* iter,
                                    const gchar**   string);

gboolean html_import_get_url_next (HtmlImport*     hi,
                                   HtmlImportIter* iter,
                                   const gchar**   string);

// =================================================================
struct _HtmlImportClass {
	GHashTable*   hash_table;
	GArray*       index_array;
	GtkListStore* list_store;
};

void html_import_class_init (HtmlImportClass* hic);
void html_import_class_clear (HtmlImportClass* hic);

void html_import_class_add (HtmlImportClass* hic,
                            const gchar* class_name);


// =================================================================

struct _HtmlImportElement {
	gchar* element_name;
	gchar* attr_name;
	gchar* label_name;

	// save all URLs for this element
	GtkListStore* url_store;

	HtmlImportClass  extension;
	HtmlImportClass  address;
};

HtmlImportElement* html_import_element_new (gchar* element_name,
                                            gchar* attr_name,
                                            gchar* label_name);

void html_import_element_destroy (HtmlImportElement* he);

void html_import_element_add_url (HtmlImportElement* he,
                                  const gchar* url);

void html_import_element_apply_selection (HtmlImportElement* hie);

#ifdef __cplusplus
}
#endif

#endif  // End of __HTML_IMPORT_H
