#include "html_import_dialog.h"
#include "selection_view.h"
#include "selection_store.h"
#include "urlgfe.h"

// signal handler --------------------------------------------------------------
static void on_choose(GtkWidget* w, HtmlImportDialog* hid)
{
	HtmlImportElement* hie;
	HtmlImportElementDialog* hied;
	gint   n_page;

	n_page = gtk_notebook_get_current_page (GTK_NOTEBOOK(hid->notebook));

	hie = html_import_get_element (hid->html_import, n_page);

	hied = html_import_element_dialog_new (hie, GTK_WINDOW(hid->self));

	if (gtk_dialog_run (hied->self)==GTK_RESPONSE_OK)
		html_import_element_apply_selection (hie);

	html_import_element_dialog_destroy (hied);
}

static void on_select_all(GtkWidget* w, HtmlImportDialog* hid)
{
	HtmlImportElement* hie;
	gint   n_page;

	n_page = gtk_notebook_get_current_page (GTK_NOTEBOOK(hid->notebook));

	hie = html_import_get_element (hid->html_import, n_page);

	selection_store_set_selection_all (hie->url_store, TRUE);
}

static void on_select_none(GtkWidget* w, HtmlImportDialog* hid)
{
	HtmlImportElement* hie;
	gint   n_page;

	n_page = gtk_notebook_get_current_page (GTK_NOTEBOOK(hid->notebook));

	hie = html_import_get_element (hid->html_import, n_page);

	selection_store_set_selection_all (hie->url_store, FALSE);
}

void html_import_dialog_add_page (HtmlImportDialog*  hid,
                                  HtmlImportElement* hie);

HtmlImportDialog* html_import_dialog_new (HtmlImport* hi,
									      GtkWindow* parent)
{
	HtmlImportDialog* hid = g_malloc (sizeof (HtmlImportDialog));
	GList*   node;

	GtkWidget* action_area;
	GtkWidget* vbox;
	GtkWidget* hbox;
	GtkWidget* label;
	GtkWidget* button;
	GtkWidget* separator;

	hid->html_import = hi;

	hid->self = (GtkDialog*)gtk_dialog_new_with_buttons (_("Import HTML file"),
	                                           parent,
	        GTK_DIALOG_MODAL | GTK_DIALOG_NO_SEPARATOR | GTK_DIALOG_DESTROY_WITH_PARENT,
	                                           NULL);
	vbox = GTK_DIALOG (hid->self)->vbox;
	action_area = GTK_DIALOG (hid->self)->action_area;

	hid->notebook = gtk_notebook_new ();
	separator = gtk_hseparator_new ();

	// setup dialog & vbox
	gtk_window_set_default_size (GTK_WINDOW(hid->self), 450, 350);
	gtk_container_set_border_width (GTK_CONTAINER(hid->self), 5);
	gtk_box_set_spacing (GTK_BOX(vbox), 5);

	// action_area ( GtkHButtonBox )
	button = gtk_button_new_with_label (_("Select none"));
	gtk_box_pack_start (GTK_BOX(action_area), button, FALSE, FALSE, 0);
	g_signal_connect (G_OBJECT(button), "clicked",
	                  G_CALLBACK(on_select_none), (gpointer)hid);

	button = gtk_button_new_with_label (_("Select all"));
	gtk_box_pack_start (GTK_BOX(action_area), button, FALSE, FALSE, 0);
	g_signal_connect (G_OBJECT(button), "clicked",
	                  G_CALLBACK(on_select_all), (gpointer)hid);

	button = gtk_button_new_with_label (_("Choose"));
	gtk_box_pack_start (GTK_BOX(action_area), button, FALSE, FALSE, 0);
	g_signal_connect (G_OBJECT(button), "clicked", 
	                  G_CALLBACK(on_choose), (gpointer)hid);

	// response buttons
	gtk_dialog_add_button (GTK_DIALOG(hid->self),
	                       GTK_STOCK_CANCEL,
	                       GTK_RESPONSE_CANCEL);
	gtk_dialog_add_button (GTK_DIALOG(hid->self),
	                       GTK_STOCK_OK,
	                       GTK_RESPONSE_OK);

	// vbox
	hbox = gtk_hbox_new (FALSE, 5);
	hid->entry = gtk_entry_new ();
	label = gtk_label_new_with_mnemonic (_("_Base href"));
	gtk_label_set_mnemonic_widget (GTK_LABEL(label), hid->entry);
	gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(hbox), hid->entry, TRUE, TRUE, 0);

	gtk_box_pack_start (GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(vbox), separator, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(vbox), hid->notebook, TRUE, TRUE, 0);

	if (hi->base_href)
		gtk_entry_set_text (GTK_ENTRY (hid->entry), hi->base_href);

	// add element
	for (node=hi->element_list; node; node=node->next)
		html_import_dialog_add_page (hid, node->data);

	gtk_widget_show_all (vbox);

	return hid;
}

void html_import_dialog_destroy (HtmlImportDialog* hid)
{
	gtk_widget_destroy (GTK_WIDGET (hid->self));
	g_free (hid);
}

gint html_import_dialog_run (HtmlImportDialog* hid)
{
	gint   response;

	response = gtk_dialog_run (hid->self);

	g_free (hid->html_import->base_href);
	hid->html_import->base_href = g_strdup (gtk_entry_get_text (GTK_ENTRY (hid->entry)));

	return response;
}

void html_import_dialog_add_page (HtmlImportDialog*  hid,
                                  HtmlImportElement* hie)
{
	GtkWidget*   label;
	GtkWidget*   scrolled_window;
	GtkTreeView* sv;

	sv = selection_view_new (hie->url_store, _("URL"));
	label = gtk_label_new_with_mnemonic (hie->label_name);

	// wrap tree_view with scrolled_window
	scrolled_window = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(scrolled_window),
	                                GTK_POLICY_AUTOMATIC,
	                                GTK_POLICY_AUTOMATIC);

	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW(scrolled_window),
	                                     GTK_SHADOW_IN);

	gtk_container_set_border_width ((GtkContainer*)scrolled_window, 5);

	gtk_container_add ((GtkContainer*)scrolled_window,
	                   GTK_WIDGET(sv) );

	gtk_notebook_append_page (GTK_NOTEBOOK(hid->notebook),
	                          scrolled_window, label);
}

// =================================================================

static void selection_store_select_all (GtkListStore* list_store)
{
	selection_store_set_selection_all (list_store, TRUE);
}

static void selection_store_select_none (GtkListStore* list_store)
{
	selection_store_set_selection_all (list_store, FALSE);
}

HtmlImportElementDialog* html_import_element_dialog_new (HtmlImportElement* hie,
                                                         GtkWindow* parent)
{
	HtmlImportElementDialog* hied;
	GtkWidget* hbox;
	GtkWidget* vbox;
	GtkWidget* vbox_sn;
	GtkWidget* hbox_sn;
	GtkWidget* scrolled_window;


	hied = g_malloc (sizeof (HtmlImportElementDialog));

	hied->element = hie;

	hied->self = (GtkDialog*)gtk_dialog_new_with_buttons (_("Choose URL from class"),
	                                                      parent,
	                                                      GTK_DIALOG_MODAL, 
	                                                      GTK_STOCK_CANCEL,
	                                                      GTK_RESPONSE_CANCEL,
	                                                      GTK_STOCK_OK,
	                                                      GTK_RESPONSE_OK,
	                                                      NULL);
	vbox = GTK_DIALOG (hied->self)->vbox;
	hbox = gtk_hbox_new (TRUE, 5);
	hied->button_all_address = gtk_button_new_with_mnemonic (_("Select _All"));
	hied->button_none_address = gtk_button_new_with_mnemonic (_("Select _None"));
	hied->button_all_extension = gtk_button_new_with_mnemonic (_("Select A_ll"));
	hied->button_none_extension = gtk_button_new_with_mnemonic (_("Select Non_e"));

	// create address_view & extension_view
	hied->address_view = selection_view_new (hie->address.list_store,
	                                         _("Address"));
	hied->extension_view = selection_view_new (hie->extension.list_store,
	                                           _("Extension"));
	// setup dialog & vbox
	gtk_box_set_spacing (GTK_BOX(vbox), 5);
	gtk_window_set_default_size (GTK_WINDOW(hied->self), 420, 320);
	gtk_container_set_border_width (GTK_CONTAINER(hied->self), 5);

	// address
	scrolled_window = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
	                                GTK_POLICY_AUTOMATIC,
	                                GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolled_window),
	                                     GTK_SHADOW_IN);
	gtk_container_add (GTK_CONTAINER (scrolled_window),
	                   GTK_WIDGET (hied->address_view) );
	vbox_sn = gtk_vbox_new (FALSE, 5);
	hbox_sn = gtk_hbox_new (FALSE, 5);
	gtk_box_pack_start (GTK_BOX(vbox_sn), scrolled_window, TRUE, TRUE, 0);
	gtk_box_pack_start (GTK_BOX(hbox_sn), hied->button_all_address, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(hbox_sn), hied->button_none_address, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(vbox_sn), hbox_sn, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(hbox), vbox_sn, TRUE, TRUE, 0);

	// extension
	scrolled_window = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
	                                GTK_POLICY_AUTOMATIC,
	                                GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolled_window),
	                                     GTK_SHADOW_IN);
	gtk_container_add (GTK_CONTAINER (scrolled_window),
	                   GTK_WIDGET (hied->extension_view) );
	vbox_sn = gtk_vbox_new (FALSE, 5);
	hbox_sn = gtk_hbox_new (FALSE, 5);
	gtk_box_pack_start (GTK_BOX(vbox_sn), scrolled_window, TRUE, TRUE, 0);
	gtk_box_pack_start (GTK_BOX(hbox_sn), hied->button_all_extension, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(hbox_sn), hied->button_none_extension, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(vbox_sn), hbox_sn, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(hbox), vbox_sn, TRUE, TRUE, 0);
	gtk_box_pack_start (GTK_BOX(vbox), hbox, TRUE, TRUE, 0);

	// signal
	g_signal_connect_swapped (hied->button_all_address, "clicked",
                              G_CALLBACK (selection_store_select_all),
	                          hied->element->address.list_store);
	g_signal_connect_swapped (hied->button_none_address, "clicked",
                              G_CALLBACK (selection_store_select_none),
	                          hied->element->address.list_store);
	g_signal_connect_swapped (hied->button_all_extension, "clicked",
                              G_CALLBACK (selection_store_select_all),
	                          hied->element->extension.list_store);
	g_signal_connect_swapped (hied->button_none_extension, "clicked",
                              G_CALLBACK (selection_store_select_none),
	                          hied->element->extension.list_store);

	selection_store_select_all (hied->element->address.list_store);

	gtk_widget_show_all (GTK_WIDGET(vbox));

	return hied;
}

void html_import_element_dialog_destroy (HtmlImportElementDialog* hied)
{
	gtk_widget_destroy (GTK_WIDGET (hied->self));
	g_free (hied);
}

