#ifndef __MAIN_APP_H
#define __MAIN_APP_H

#include "main_window.h"
#include "category_store.h"
#include "global_setting.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef struct _MainApp          MainApp;

struct _MainApp {
	CategoryStore    category_store;
	MainWindow       window;

	// default setting
	GlobalSetting*   setting;

	// for monitoring clipboard
	GtkClipboard* clipboard;
	gchar*        clipboard_text;
	gint          clipboard_timer_tag;
	gboolean      clipboard_monitoring;
	gboolean      clipboard_waiting;
};

void main_app_init (MainApp* app);
void main_app_run (MainApp* app);
void main_app_destroy (MainApp* app);

void main_app_dnd_init (MainApp* app);

void main_app_apply_setting (MainApp* app);

void main_app_add_download_from_url (MainApp* app, const gchar* url_string);

gboolean main_app_get_download_from_user (MainApp* app,
                                          DownloadSetting* setting,
                                          Category** selected_category,
                                          gboolean multiple);

void main_app_move_selected_download (MainApp* app,
                                      DownloadListView* dlv_src,
                                      DownloadListView* dlv_dest);

gboolean main_app_read_setting_file (MainApp* app, const gchar* filename);
gboolean main_app_read_data_file (MainApp* app, const gchar* filename);

void main_app_connect_category_signal (MainApp* app,
                                       Category* category);

void main_app_on_save (MainApp* app);
void main_app_on_save_as (MainApp* app);
void main_app_on_load (MainApp* app);
void main_app_on_import_html (MainApp* app);
void main_app_on_export_url (MainApp* app);
gboolean main_app_on_delete_event (MainApp* app);
gboolean main_app_on_timer (MainApp* app);

void main_app_start_monitoring_clipboard (MainApp* app);
void main_app_stop_monitoring_clipboard (MainApp* app);

#ifdef __cplusplus
}
#endif

#endif  /* End of __MAIN_APP_H */

