#include "urlgfe.h"
#include "main_window.h"
#include "category_view.h"

// utility functions ------------------------------------------------

// utility for GtkTextBuffer
static void text_buffer_add_text (GtkTextBuffer* buffer,
                                  GtkTextTag* text_tag,
                                  gchar *name, gchar *str)
{
	GtkTextIter    text_iter;
	GtkTextIter    text_iter2;
	GtkTextMark*   mark;

	gtk_text_buffer_get_end_iter (buffer, &text_iter2);
	mark = gtk_text_buffer_create_mark (buffer, NULL, &text_iter2, TRUE);
	gtk_text_buffer_insert (buffer, &text_iter2, name, -1);
	// apply tag
	gtk_text_buffer_get_end_iter (buffer, &text_iter2);
	gtk_text_buffer_get_iter_at_mark (buffer, &text_iter, mark);
	gtk_text_buffer_apply_tag (buffer, text_tag,
	                           &text_iter, &text_iter2);
	gtk_text_buffer_delete_mark (buffer, mark);

	gtk_text_buffer_insert (buffer, &text_iter2,
	                        STR_N2E(str), -1);
	// to next line
	gtk_text_buffer_get_end_iter (buffer, &text_iter2);
	gtk_text_buffer_insert (buffer, &text_iter2, "\n", -1);
}

// utility for create button
static GtkWidget* tool_button_new (const gchar* stock_id,
                                   const gchar* tip_str,
								   GtkTooltips* tips)
{
	GtkWidget* button;
	GtkWidget* image;

	button = gtk_button_new();
	image = gtk_image_new_from_stock (stock_id,
	                                  GTK_ICON_SIZE_SMALL_TOOLBAR);

	gtk_container_add (GTK_CONTAINER(button),
	                   image);
	gtk_button_set_relief (GTK_BUTTON (button),
	                       GTK_RELIEF_NONE);
	GTK_WIDGET_UNSET_FLAGS (button,
	                        GTK_CAN_FOCUS);

	gtk_tooltips_set_tip (tips, button,
	                      tip_str, NULL);

	return button;
}

// utility for GtkStatusbar
static void statusbar_set_n_selected (GtkStatusbar* sb, guint n_selected)
{
	static guint last_id=0;
	gchar* string;

	if (last_id)
		gtk_statusbar_pop (sb, last_id);

	string = g_strdup_printf (_(" Selected %d files"), n_selected);
	last_id = gtk_statusbar_get_context_id (sb, string);
	gtk_statusbar_push (sb, last_id, string);
	g_free (string);
}
// End of utility functions =========================================


// download view and store signal handler ---------------------------

// callback for gtk_tree_selection_set_select_function ()
static gboolean on_download_selected (GtkTreeSelection *selection,
                                      GtkTreeModel *model,
                                      GtkTreePath *path,
                                      gboolean path_currently_selected,
                                      MainWindow* mw)
{
	GtkTreeIter iter;
	GtkTextBuffer*   textbuf;
	Download*        dl_data;

	gtk_tree_model_get_iter (model, &iter, path);

	textbuf = gtk_text_view_get_buffer (mw->text_view);
	gtk_text_buffer_set_text (textbuf, "", 0);

	if (mw->current_category==NULL)
		return TRUE;
//	if (path_currently_selected || mw->current_category==NULL)
//		return TRUE;

	gtk_tree_model_get (model, &iter,
	                    DOWNLOAD_COL_DATA,
	                    &dl_data,
	                    -1);
	main_window_text_view_show (mw, dl_data);

	return TRUE;
}

static gboolean on_download_button_event (GtkWidget* widget,
                                          GdkEventButton* event,
                                          MainWindow* mw)
{
	if (event->type==GDK_BUTTON_PRESS && event->button==3) {
		// press right button
		gtk_menu_popup (mw->download_menu.self,
		                NULL,
		                NULL,
		                NULL,
		                NULL,
		                0,
		                gtk_get_current_event_time());
		return TRUE;
	}
	main_window_decide_download_sensitive (mw);
	return FALSE;
}

static void on_download_store_row_changed (GtkTreeModel* treemodel,
                                           GtkTreePath*  path,
                                           GtkTreeIter*  iter,
                                           MainWindow*   mw)
{
	main_window_text_view_decide_update (mw, path);
}
// End of download view & store signal handler ======================

// category view signal handler -------------------------------------
static gboolean on_category_button_event (GtkWidget* widget,
                                          GdkEventButton* event,
                                          MainWindow* mw)
{
	if (event->type==GDK_BUTTON_RELEASE && event->button==3) {
		// press right button
		gtk_menu_popup (mw->category_menu.self,
		                NULL,
		                NULL,
		                NULL,
		                NULL,
		                0,
		                gtk_get_current_event_time());
		return TRUE;
	}
	return FALSE;
}
// End of category view signal handler ==============================


// Menu
void file_menu_init (FileMenu* fm)
{
	GtkWidget* image;

	fm->self = (GtkMenu*)gtk_menu_new ();
	fm->load = gtk_image_menu_item_new_with_mnemonic (_("_Load..."));
	image = gtk_image_new_from_stock (GTK_STOCK_OPEN, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image ((GtkImageMenuItem*)fm->load, image);

	fm->save = gtk_image_menu_item_new_with_mnemonic (_("_Save"));
	image = gtk_image_new_from_stock (GTK_STOCK_SAVE, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image ((GtkImageMenuItem*)fm->save, image);

	fm->save_as = gtk_image_menu_item_new_with_mnemonic (_("Save _as..."));
	image = gtk_image_new_from_stock (GTK_STOCK_SAVE, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image ((GtkImageMenuItem*)fm->save_as, image);

	fm->import_html = gtk_image_menu_item_new_with_mnemonic (_("_Import from HTML file..."));
	image = gtk_image_new_from_stock (GTK_STOCK_CONVERT, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image ((GtkImageMenuItem*)fm->import_html, image);

	fm->export = gtk_image_menu_item_new_with_mnemonic (_("_Export URL..."));
	image = gtk_image_new_from_stock (GTK_STOCK_CONVERT, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image ((GtkImageMenuItem*)fm->export, image);

	gtk_menu_append (fm->self, fm->load);
	gtk_menu_append (fm->self, fm->save);
	gtk_menu_append (fm->self, fm->save_as);
	gtk_menu_append (fm->self, gtk_separator_menu_item_new() );
	gtk_menu_append (fm->self, fm->import_html);
	gtk_menu_append (fm->self, fm->export);
}

void global_menu_init (GlobalMenu* gm)
{
	GtkWidget* image;

	gm->self = (GtkMenu*)gtk_menu_new ();

	gm->start_job = gtk_image_menu_item_new_with_mnemonic (_("St_art action"));
	image = gtk_image_new_from_stock (GTK_STOCK_EXECUTE, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image ((GtkImageMenuItem*)gm->start_job, image);
	gm->stop_all = gtk_image_menu_item_new_with_mnemonic (_("_Stop all"));
	image = gtk_image_new_from_stock (GTK_STOCK_STOP, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image ((GtkImageMenuItem*)gm->stop_all, image);

	gm->category_default = gtk_menu_item_new_with_mnemonic (_("_Category default..."));
	gm->setting = gtk_menu_item_new_with_mnemonic (_("_Global Setting..."));

	gtk_menu_append (gm->self, gm->start_job);
	gtk_menu_append (gm->self, gm->stop_all);
	gtk_menu_append (gm->self, gtk_separator_menu_item_new() );
	gtk_menu_append (gm->self, gm->category_default);
	gtk_menu_append (gm->self, gm->setting);
}

void category_menu_init (CategoryMenu* cm)
{
	GtkWidget* image;

	cm->self = (GtkMenu*)gtk_menu_new ();

	cm->new_category = gtk_image_menu_item_new_with_mnemonic (_("_New"));
	image = gtk_image_new_from_stock (GTK_STOCK_NEW, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image ((GtkImageMenuItem*)cm->new_category, image);

	cm->erase_category = gtk_image_menu_item_new_with_mnemonic (_("_Erase"));
	image = gtk_image_new_from_stock (GTK_STOCK_CLOSE, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image ((GtkImageMenuItem*)cm->erase_category, image);

	cm->properties = gtk_image_menu_item_new_with_mnemonic (_("_Properties..."));
	image = gtk_image_new_from_stock (GTK_STOCK_PROPERTIES, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image ((GtkImageMenuItem*)cm->properties ,image);

//	menu.append( gtk_tearoff_menu_item_new() );
	gtk_menu_append (cm->self, cm->new_category);
	gtk_menu_append (cm->self, cm->erase_category);
	gtk_menu_append (cm->self, cm->properties);
}

void download_menu_init (DownloadMenu* dm)
{
	GtkWidget* image;

	dm->self = (GtkMenu*)gtk_menu_new ();

	dm->new_download = gtk_image_menu_item_new_with_mnemonic (_("_New download..."));
	image = gtk_image_new_from_stock (GTK_STOCK_NEW, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image ((GtkImageMenuItem*)dm->new_download, image);

	dm->add_batch = gtk_image_menu_item_new_with_mnemonic (_("_Add batch download..."));
	image = gtk_image_new_from_stock (GTK_STOCK_SORT_ASCENDING, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image ((GtkImageMenuItem*)dm->add_batch, image);

	dm->erase = gtk_image_menu_item_new_with_mnemonic(_("_Erase"));
	image = gtk_image_new_from_stock(GTK_STOCK_DELETE, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image ((GtkImageMenuItem*)dm->erase, image);

	dm->move_to = gtk_image_menu_item_new_with_mnemonic(_("Move t_o..."));
	image = gtk_image_new_from_stock(GTK_STOCK_DND_MULTIPLE, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image ((GtkImageMenuItem*)dm->move_to, image);

	dm->move_top = gtk_image_menu_item_new_with_mnemonic(_("Move _top"));
	image = gtk_image_new_from_stock(GTK_STOCK_GOTO_TOP, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image ((GtkImageMenuItem*)dm->move_top, image);

	dm->move_bottom = gtk_image_menu_item_new_with_mnemonic(_("Move _bottom"));
	image = gtk_image_new_from_stock(GTK_STOCK_GOTO_BOTTOM, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image ((GtkImageMenuItem*)dm->move_bottom, image);

	dm->start = gtk_image_menu_item_new_with_mnemonic(_("Start automatically"));
	image = gtk_image_new_from_stock(GTK_STOCK_YES, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image ((GtkImageMenuItem*)dm->start, image);

	dm->stop  = gtk_image_menu_item_new_with_mnemonic(_("Stop"));
	image = gtk_image_new_from_stock(GTK_STOCK_NO, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image ((GtkImageMenuItem*)dm->stop ,image);

	dm->properties = gtk_image_menu_item_new_with_mnemonic(_("_Properties..."));
	image = gtk_image_new_from_stock(GTK_STOCK_PROPERTIES, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image ((GtkImageMenuItem*)dm->properties, image);

//	menu.append( gtk_tearoff_menu_item_new() );
	gtk_menu_append (dm->self, dm->new_download);
	gtk_menu_append (dm->self, dm->add_batch);
	gtk_menu_append (dm->self, dm->erase);
	gtk_menu_append (dm->self, gtk_separator_menu_item_new() );
	gtk_menu_append (dm->self, dm->move_to);
	gtk_menu_append (dm->self, dm->move_top);
	gtk_menu_append (dm->self, dm->move_bottom);
	gtk_menu_append (dm->self, gtk_separator_menu_item_new() );
	gtk_menu_append (dm->self, dm->start);
	gtk_menu_append (dm->self, dm->stop);
	gtk_menu_append (dm->self, dm->properties);
}

void help_menu_init (HelpMenu* hm)
{
	GtkWidget* image;

	hm->self = (GtkMenu*)gtk_menu_new ();
	hm->about_urlgfe = gtk_image_menu_item_new_with_mnemonic (_("_About UrlGfe..."));
	image = gtk_image_new_from_stock (GTK_STOCK_DIALOG_INFO, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image ((GtkImageMenuItem*)hm->about_urlgfe, image);

	gtk_menu_append (hm->self, hm->about_urlgfe);
}



// MainWindow
void main_window_init_toolbar (MainWindow* wnd,
                               GtkWidget* hbox)
{
	GtkTooltips* tips;

	tips = gtk_tooltips_new ();

	// button Start Download
	wnd->button_start = tool_button_new (GTK_STOCK_EXECUTE,
	                                     _("Begin downloading"),
	                                     tips);
	gtk_box_pack_start (GTK_BOX(hbox), wnd->button_start,
	                    FALSE, FALSE, 1);

	// button Stop Download
	wnd->button_stop = tool_button_new (GTK_STOCK_STOP,
	                                    _("Stop downloading"),
	                                    tips);
	gtk_box_pack_start (GTK_BOX(hbox), wnd->button_stop,
	                    FALSE, FALSE, 1);

	// button New Download
	wnd->button_new = tool_button_new (GTK_STOCK_NEW,
	                                   _("Create new download"),
	                                   tips);
	gtk_box_pack_start (GTK_BOX(hbox), wnd->button_new,
	                    FALSE, FALSE, 1);

	// button Add batch
	wnd->button_batch = tool_button_new (GTK_STOCK_SORT_ASCENDING,
	                                     _("Add batch download"),
	                                     tips);
	gtk_box_pack_start (GTK_BOX(hbox), wnd->button_batch,
	                    FALSE, FALSE, 1);

	// button Delete
	wnd->button_erase = tool_button_new (GTK_STOCK_DELETE,
	                                     _("Delete selected jobs"),
	                                     tips);
	gtk_box_pack_start (GTK_BOX(hbox), wnd->button_erase,
	                    FALSE, FALSE, 1);

	// button Property
	wnd->button_property = tool_button_new (GTK_STOCK_PROPERTIES,
	                                        _("Properties"),
	                                        tips);
	gtk_box_pack_start (GTK_BOX(hbox), wnd->button_property,
	                    FALSE, FALSE, 1);

	// button Move top
	wnd->button_move_top = tool_button_new (GTK_STOCK_GOTO_TOP,
	                                        _("Move top"),
	                                        tips);
	gtk_box_pack_start (GTK_BOX(hbox), wnd->button_move_top,
	                    FALSE, FALSE, 1);

	// button Move bottom
	wnd->button_move_bottom = tool_button_new (GTK_STOCK_GOTO_BOTTOM,
	                                           _("Move bottom"),
	                                           tips);
	gtk_box_pack_start (GTK_BOX(hbox), wnd->button_move_bottom,
	                    FALSE, FALSE, 1);

	gtk_widget_show_all (hbox);
}

void main_window_init (MainWindow* mw, CategoryStore* c_store)
{
	GtkScrolledWindow* scrolled;
	GtkScrolledWindow* scrolled_text;
	GtkWidget*  hbox;
	GtkWidget*  menu_item;
	GtkTextBuffer *buffer;

	mw->current_category = NULL;
	mw->current_download_view = NULL;
	mw->category_store = c_store;
	mw->vbox = gtk_vbox_new (FALSE, 3);
	mw->self = (GtkWindow*)gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title (mw->self, "UrlGfe");
	gtk_window_set_default_size (mw->self, 515, 350);

	// menu bar --------------------------------------------------
	mw->menu_bar = gtk_menu_bar_new ();

	// file menu
	file_menu_init (&mw->file_menu);
	menu_item = gtk_menu_item_new_with_mnemonic (_("_File"));
	gtk_menu_item_set_submenu ((GtkMenuItem*)menu_item,
	                           (GtkWidget*)mw->file_menu.self);
	gtk_menu_shell_append ((GtkMenuShell *)mw->menu_bar, menu_item);

	// global menu
	global_menu_init (&mw->global_menu);
	menu_item = gtk_menu_item_new_with_mnemonic (_("_Global"));
	gtk_menu_shell_append ((GtkMenuShell *)mw->menu_bar, menu_item);
	gtk_menu_item_set_submenu ((GtkMenuItem*)menu_item,
	                           (GtkWidget*)mw->global_menu.self);

	// category menu
	category_menu_init (&mw->category_menu);
	menu_item = gtk_menu_item_new_with_mnemonic (_("_Category"));
	gtk_menu_item_set_submenu ((GtkMenuItem*)menu_item,
	                           (GtkWidget*)mw->category_menu.self);
	gtk_menu_shell_append ((GtkMenuShell *)mw->menu_bar, menu_item);

	// download menu
	download_menu_init (&mw->download_menu);
	menu_item = gtk_menu_item_new_with_mnemonic(_("_Download"));
	gtk_menu_item_set_submenu ((GtkMenuItem*)menu_item,
	                           (GtkWidget*)mw->download_menu.self);
	gtk_menu_shell_append ((GtkMenuShell *)mw->menu_bar, menu_item);

	// about
	help_menu_init (&mw->help_menu);
	menu_item = gtk_menu_item_new_with_mnemonic(_("_Help"));
	gtk_menu_item_set_submenu ((GtkMenuItem*)menu_item,
	                           (GtkWidget*)mw->help_menu.self);
	gtk_menu_shell_append ((GtkMenuShell *)mw->menu_bar, menu_item);
	// End of menu bar ==============================================

	// category view
	mw->category_view = category_tree_view_new (c_store);
	gtk_tree_view_set_headers_visible (mw->category_view, FALSE);
	g_signal_connect (mw->category_view, "button-release-event",
	                  G_CALLBACK(on_category_button_event), (gpointer)mw);
	g_signal_connect_swapped (mw->category_view,
	                          "cursor-changed",
	                          G_CALLBACK(main_window_selection_changed),
	                          (gpointer)mw);
	gtk_widget_set_size_request ((GtkWidget*)mw->category_view, 155, 100);
	// create ScrolledWindow for CategoryView
	scrolled = (GtkScrolledWindow*)gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_policy (scrolled,
	                                GTK_POLICY_AUTOMATIC,
	                                GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type (scrolled,
	                                     GTK_SHADOW_IN);
	gtk_container_add (GTK_CONTAINER (scrolled),
	                   GTK_WIDGET (mw->category_view));

	// create and setup TextView
	mw->text_view = (GtkTextView*)gtk_text_view_new ();
	gtk_text_view_set_left_margin (mw->text_view, 2);
	gtk_text_view_set_right_margin (mw->text_view, 2);
	gtk_text_view_set_editable (mw->text_view, FALSE);
	gtk_text_view_set_cursor_visible (mw->text_view, FALSE);
	gtk_widget_set_size_request ((GtkWidget*)mw->text_view, 35, 35);
	buffer = gtk_text_view_get_buffer (mw->text_view);
	mw->text_tag = gtk_text_buffer_create_tag (buffer, "blue_foreground",
	                                           "foreground", "blue", NULL);
	// create ScrolledWindow for TextView
	scrolled_text = (GtkScrolledWindow*)gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_policy (scrolled_text,
	                                GTK_POLICY_AUTOMATIC,
	                                GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type (scrolled_text,
	                                     GTK_SHADOW_IN);
	gtk_container_add (GTK_CONTAINER (scrolled_text),
	                   GTK_WIDGET (mw->text_view));


	// status bar
	mw->statusbar = (GtkStatusbar*) gtk_statusbar_new ();
	statusbar_set_n_selected (mw->statusbar, 0);

	// toolbar
	hbox = gtk_hbox_new (FALSE, 0);
	main_window_init_toolbar (mw, hbox);
	mw->vbox_download = gtk_vbox_new (FALSE, 2);
	gtk_box_pack_start (GTK_BOX(mw->vbox_download),
	                    GTK_WIDGET(hbox),
						FALSE, FALSE, 0);

	mw->hpaned = (GtkPaned*)gtk_hpaned_new ();
	mw->vpaned = (GtkPaned*)gtk_vpaned_new ();

	gtk_paned_pack1 (mw->hpaned,
	                 (GtkWidget*)scrolled,
	                 FALSE, TRUE);
	gtk_paned_add2 (mw->hpaned,
	                (GtkWidget*)mw->vpaned);

	gtk_paned_pack1 (mw->vpaned,
	                 (GtkWidget*)mw->vbox_download,
					 TRUE, FALSE);
	gtk_paned_add2 (mw->vpaned,
	                (GtkWidget*)scrolled_text);

	gtk_box_pack_start (GTK_BOX (mw->vbox), mw->menu_bar,
	                    FALSE, FALSE, 0);

	gtk_box_pack_start (GTK_BOX (mw->vbox), GTK_WIDGET (mw->hpaned),
	                    TRUE, TRUE, 0);

	gtk_box_pack_start (GTK_BOX (mw->vbox), GTK_WIDGET (mw->statusbar),
	                    FALSE, FALSE, 0);

	gtk_container_add (GTK_CONTAINER (mw->self),
	                   GTK_WIDGET (mw->vbox));

	gtk_widget_show_all ((GtkWidget*)mw->vbox);
}

void main_window_connect_download_view (MainWindow* mw,
                                        DownloadListView* dlv)
{
	GtkTreeModel* model;
	GtkTreeSelection* selection;

	model = gtk_tree_view_get_model (dlv->self);

	g_signal_connect_swapped (dlv->self, "cursor-changed",
	                          G_CALLBACK(main_window_decide_download_sensitive),
	                          mw);
	g_signal_connect (dlv->self, "button-press-event",
	                  G_CALLBACK(on_download_button_event),
	                  mw);
	g_signal_connect (dlv->self, "button-release-event",
	                  G_CALLBACK(on_download_button_event),
	                  mw);
	g_signal_connect_swapped (model, "row-deleted",
	                          G_CALLBACK(main_window_decide_download_sensitive),
	                          mw);
	selection = gtk_tree_view_get_selection (dlv->self);

	g_signal_connect (dlv->store->list_store, "row-changed",
	                  G_CALLBACK(on_download_store_row_changed),
	                  mw);

	gtk_tree_selection_set_select_function (selection,
	                              (GtkTreeSelectionFunc)on_download_selected,
	                              mw, NULL);
}

void main_window_set_download_view (MainWindow* mw, DownloadListView* dlv)
{
	GtkWidget* current_view;

	if (mw->current_download_view) {
		current_view = (GtkWidget*)mw->current_download_view->container;

		g_object_ref (current_view);

		gtk_container_remove ((GtkContainer*)mw->vbox_download,
		                      current_view);
	}
	if (dlv) {
		gtk_box_pack_start ((GtkBox*)mw->vbox_download,
		                    (GtkWidget*)dlv->container,
							TRUE, TRUE, 0);
	}
	mw->current_download_view = dlv;
	main_window_text_view_update (mw);
}

void main_window_update_download_view (MainWindow* mw)
{
	if (mw->current_download_view) {
		gtk_widget_queue_draw (GTK_WIDGET (mw->current_download_view->self));
	}
}

void main_window_decide_download_sensitive (MainWindow* window)
{
	GtkTreeSelection* selection;
	guint n_selected;

	if (window->current_download_view) {
		selection = gtk_tree_view_get_selection (
		                window->current_download_view->self);

		n_selected = gtk_tree_selection_count_selected_rows (selection);
		statusbar_set_n_selected (window->statusbar, n_selected);
		if (n_selected) {
			main_window_set_download_sensitive (window, TRUE);
			return;
		}
	}
	main_window_set_download_sensitive (window, FALSE);
}

void main_window_set_download_sensitive (MainWindow* mw, gboolean sens)
{
	// tool button
	gtk_widget_set_sensitive (mw->button_start, sens);
	gtk_widget_set_sensitive (mw->button_stop, sens);
	gtk_widget_set_sensitive (mw->button_erase, sens);
	gtk_widget_set_sensitive (mw->button_property, sens);
	gtk_widget_set_sensitive (mw->button_move_top, sens);
	gtk_widget_set_sensitive (mw->button_move_bottom, sens);
	// menu item
	gtk_widget_set_sensitive (mw->download_menu.start, sens);
	gtk_widget_set_sensitive (mw->download_menu.stop, sens);
	gtk_widget_set_sensitive (mw->download_menu.erase, sens);
	gtk_widget_set_sensitive (mw->download_menu.move_to, sens);
	gtk_widget_set_sensitive (mw->download_menu.properties, sens);
	gtk_widget_set_sensitive (mw->download_menu.move_top, sens);
	gtk_widget_set_sensitive (mw->download_menu.move_bottom, sens);
}

void main_window_set_category_sensitive (MainWindow* mw, gboolean sens)
{
	// tool button
	gtk_widget_set_sensitive (mw->button_new, sens);
	gtk_widget_set_sensitive (mw->button_batch, sens);
	// file menu item
	gtk_widget_set_sensitive (mw->file_menu.export, sens);
	gtk_widget_set_sensitive (mw->file_menu.import_html, sens);
	gtk_widget_set_sensitive (mw->file_menu.save_as, sens);
	// download menu item
	gtk_widget_set_sensitive (mw->download_menu.new_download, sens);
	gtk_widget_set_sensitive (mw->download_menu.add_batch, sens);
	// category menu item
	gtk_widget_set_sensitive (mw->category_menu.erase_category, sens);
	gtk_widget_set_sensitive (mw->category_menu.properties, sens);
}

// signal "cursor-changed" connect here
void main_window_selection_changed (MainWindow* window)
{
	GtkTreeIter       iter;
	GtkTreeModel*     model;
	GtkTreeSelection* selection;
	DownloadListView* download_view;
	gboolean          had_selected;

	selection = gtk_tree_view_get_selection (window->category_view);
	had_selected = gtk_tree_selection_get_selected (selection,
	                                                &model,
	                                                &iter);

	if (had_selected) {
		gtk_tree_model_get (model,
		                    &iter,
		                    CATEGORY_COL_DATA,
		                    &window->current_category,
		                    CATEGORY_COL_DOWNLOAD_VIEW,
		                    &download_view,
		                    -1);

		main_window_set_download_view (window, download_view);
		// change sensitive state
		main_window_set_category_sensitive (window, TRUE);
		main_window_decide_download_sensitive (window);
	}
	else {
		window->current_category = NULL;
		main_window_set_category_sensitive (window, FALSE);
		main_window_set_download_sensitive (window, FALSE);
	}
}

void main_window_text_view_update (MainWindow* mw)
{
	GtkTextBuffer*   textbuf;

	textbuf = gtk_text_view_get_buffer (mw->text_view);
	gtk_text_buffer_set_text (textbuf, "", 0);
	main_window_text_view_decide_update (mw, NULL);
}

void main_window_text_view_decide_update (MainWindow* mw, GtkTreePath* path)
{
    Download*         dl_data;
	GtkTreeSelection* selection;
	GtkTreeModel*     model;
	GtkTreeIter       iter;
	GList*            path_list;
	gint              n_selected;

	if (mw->current_download_view == NULL)
		return;

	selection = gtk_tree_view_get_selection (mw->current_download_view->self);
	model = gtk_tree_view_get_model (mw->current_download_view->self);

	download_store_lock (mw->current_download_view->store);

	n_selected = gtk_tree_selection_count_selected_rows (selection);

	if (n_selected==1) {
		path_list = gtk_tree_selection_get_selected_rows (selection, NULL);
		if (path==NULL || gtk_tree_path_compare (path_list->data, path)==0)
		{
			if (gtk_tree_model_get_iter (model, &iter, path_list->data)) {
				gtk_tree_model_get (model, &iter,
				                    DOWNLOAD_COL_DATA,
				                    &dl_data,
				                    -1);
				main_window_text_view_show (mw, dl_data);
			}
		}
		g_list_foreach (path_list, (GFunc)gtk_tree_path_free, NULL);
		g_list_free (path_list);
	}
	download_store_unlock (mw->current_download_view->store);
}

void main_window_text_view_show (MainWindow* mw, Download* dl_data)
{
	GtkTextBuffer*   textbuf;
	DownloadSetting* setting = DOWNLOAD_SETTING (dl_data);

	textbuf = gtk_text_view_get_buffer (mw->text_view);
	gtk_text_buffer_set_text (textbuf, "", 0);

	if (dl_data==NULL)
		return;

	if (CATEGORY_SETTING (mw->current_category)->text_filename) {
		text_buffer_add_text (textbuf, mw->text_tag,
		                      _("Filename : "), setting->filename);
	}
	if (CATEGORY_SETTING (mw->current_category)->text_directory) {
		text_buffer_add_text (textbuf, mw->text_tag,
		                      _("Directory : "), setting->directory);
	}
	if (CATEGORY_SETTING (mw->current_category)->text_url) {
		text_buffer_add_text (textbuf, mw->text_tag,
		                      _("URL : "), setting->url);
	}
	if (CATEGORY_SETTING (mw->current_category)->text_message) {
		text_buffer_add_text (textbuf, mw->text_tag,
		                      _("Last Message : "),
		                      (dl_data->message) ? dl_data->message : _("None"));
	}
}

void main_window_on_category_erasing (MainWindow* mw, Category* erasing)
{
	Category*  next_category;
	DownloadListView*  dlv = NULL;
	GtkTreeSelection*  selection;

	next_category = category_store_get_next (mw->category_store, erasing);

	if (next_category) {
		selection = gtk_tree_view_get_selection (mw->category_view);
		gtk_tree_selection_select_iter (selection, &next_category->iter_tree_store);
		dlv = &next_category->waiting_view;
	}

	main_window_set_download_view (mw, dlv);
}

