//  proxy_setting.h
//
//  Plus Huang

#include <string.h>
#include <stdlib.h>
#include "proxy_setting.h"
#include "urlgfe.h"

void setting_set_string (gchar** str_addr, const gchar* src)
{
	gchar* old_str = *str_addr;

	*str_addr = (src!=NULL && *src!=0)? g_strdup (src) : NULL;
	g_free(old_str);
}

void setting_set_string_len (gchar** str_addr, const gchar* src, int len)
{
	gchar* old_str = *str_addr;

	*str_addr = (src!=NULL && *src!=0)? g_strndup (src, len) : NULL;
	g_free(old_str);
}


/* ProxySetting ============================================================ */
ProxySetting* proxy_setting_init (ProxySetting* pa)
{
	pa->type = PROXY_TYPE_NONE;
	pa->authentication = FALSE;
	pa->port = 80;
	pa->address = NULL;
	pa->username = NULL;
	pa->password = NULL;

	return pa;
}

void proxy_setting_clear (ProxySetting* pa)
{
	g_free (pa->address);
	g_free (pa->username);
	g_free (pa->password);
	pa->address = NULL;
	pa->username = NULL;
	pa->password = NULL;
}

void proxy_setting_destroy (ProxySetting* pa)
{
	proxy_setting_clear (pa);
	g_free (pa);
}

void proxy_setting_assign (ProxySetting* pa, ProxySetting* src)
{
	pa->type           = src->type;
	pa->authentication = src->authentication;
	pa->port           = src->port;
	proxy_setting_set_address(pa, src->address);
	proxy_setting_set_username(pa, src->username);
	proxy_setting_set_password(pa, src->password);
}

void proxy_setting_write_xml (ProxySetting* setting,
                              FILE* file, int level)
{
	fprintf (file,
	         "%*c<" PROXY_SETTING_TAG " "         // Start of format string
	         "type=\"%d\" "
	         "authentication=\"%d\" "
	         "port=\"%d\" "
	         "address=\"%s\" "
	         "username=\"%s\" "
	         "password=\"%s\" />\n",   // End of format string
	         level, ' ',
	         setting->type,
	         setting->authentication,
	         setting->port,
	         STR_N2E (setting->address),
	         STR_N2E (setting->username),
	         STR_N2E (setting->password)
	         );
}

