#include "proxy_setting_view.h"
#include "urlgfe.h"

static void on_toggled (ProxySettingView* view)
{
	gboolean auth_sensitive;
	gboolean proxy_sensitive;

	proxy_sensitive = !gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(view->radio_none));
	auth_sensitive = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(view->toggle_authentication));

	if (proxy_sensitive == FALSE)
		auth_sensitive = FALSE;

	gtk_widget_set_sensitive (view->label_host, proxy_sensitive);
	gtk_widget_set_sensitive (view->label_port, proxy_sensitive);
	gtk_widget_set_sensitive (view->entry_host, proxy_sensitive);
	gtk_widget_set_sensitive (view->spin_port, proxy_sensitive);
	gtk_widget_set_sensitive (view->toggle_authentication, proxy_sensitive);

	gtk_widget_set_sensitive (view->label_username, auth_sensitive);
	gtk_widget_set_sensitive (view->label_password, auth_sensitive);
	gtk_widget_set_sensitive (view->entry_username, auth_sensitive);
	gtk_widget_set_sensitive (view->entry_password, auth_sensitive);
}

void proxy_setting_view_init (ProxySettingView* view)
{
	GtkWidget* hbox;
	GtkWidget* vbox;
	GtkSizeGroup* size_group;

	// class initial base class : SettingView
	setting_view_init (SETTING_VIEW (view));

	view->self = gtk_hbox_new (FALSE, 3);

	// initial and setup widgets  -----------------------------------
	view->radio_none = gtk_radio_button_new_with_mnemonic (NULL,
	                                          _("_Don't use"));
	view->radio_http = gtk_radio_button_new_with_mnemonic_from_widget (
	                                          (GtkRadioButton*)view->radio_none,
	                                          "_HTTP");
	view->radio_socks5 = gtk_radio_button_new_with_mnemonic_from_widget (
	                                          (GtkRadioButton*)view->radio_none,
	                                          "_SOCKS v5");

	view->entry_host     = gtk_entry_new ();
	view->spin_port      = gtk_spin_button_new_with_range (0.0, 65535.0, 1.0);
	view->label_host     = gtk_label_new_with_mnemonic (_("Host:"));
	view->label_port     = gtk_label_new_with_mnemonic (":");
	gtk_entry_set_width_chars (GTK_ENTRY (view->spin_port), 4);
	gtk_label_set_mnemonic_widget (GTK_LABEL(view->label_host),
	                               view->entry_host);
	gtk_label_set_mnemonic_widget (GTK_LABEL(view->label_port),
	                               view->spin_port);

	view->toggle_authentication = gtk_check_button_new_with_mnemonic (_("Authentication"));
	view->entry_username = gtk_entry_new ();
	view->entry_password = gtk_entry_new ();
	view->label_username = gtk_label_new_with_mnemonic (_("Username:"));
	view->label_password = gtk_label_new_with_mnemonic (_("Password:"));
	gtk_entry_set_visibility (GTK_ENTRY (view->entry_password), FALSE);
	gtk_label_set_mnemonic_widget (GTK_LABEL(view->label_username),
	                               view->entry_username);
	gtk_label_set_mnemonic_widget (GTK_LABEL(view->label_password),
	                               view->entry_password);

	// pack left side widgets ---------------------------------------
	size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
	gtk_size_group_add_widget (size_group, view->radio_none);
	gtk_size_group_add_widget (size_group, view->radio_http);
	gtk_size_group_add_widget (size_group, view->radio_socks5);

	vbox = gtk_vbox_new (FALSE, 3);
	gtk_box_pack_start (GTK_BOX(vbox), view->radio_none, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(vbox), view->radio_http, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(vbox), view->radio_socks5, FALSE, FALSE, 0);
	// pack left side
	gtk_box_pack_start (GTK_BOX(view->self), vbox, FALSE, FALSE, 0);

	// pack center separator ----------------------------------------
	gtk_box_pack_start (GTK_BOX(view->self), gtk_vseparator_new(),
	                    FALSE, FALSE, 0);

	// pack right side widgets --------------------------------------
	size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
//	gtk_size_group_add_widget (size_group, view->label_host);
//	gtk_size_group_add_widget (size_group, view->label_port);
	gtk_size_group_add_widget (size_group, view->label_username);
	gtk_size_group_add_widget (size_group, view->label_password);

	vbox = gtk_vbox_new (FALSE, 3);
	// pack proxy & port
	hbox = gtk_hbox_new (FALSE, 3);
	gtk_box_pack_start (GTK_BOX(hbox), view->label_host, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(hbox), view->entry_host, TRUE, TRUE, 0);
//	gtk_box_pack_start (GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
//	hbox = gtk_hbox_new (FALSE, 3);
	gtk_box_pack_start (GTK_BOX(hbox), view->label_port, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(hbox), view->spin_port, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
/*
	// pack proxy
	hbox = gtk_hbox_new (FALSE, 3);
	gtk_box_pack_start (GTK_BOX(hbox), view->label_host, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(hbox), view->entry_host, TRUE, TRUE, 0);
	gtk_box_pack_start (GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
	// pack port
	hbox = gtk_hbox_new (FALSE, 3);
	gtk_box_pack_start (GTK_BOX(hbox), view->label_port, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(hbox), view->spin_port, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
*/
	gtk_box_pack_start (GTK_BOX(vbox), view->toggle_authentication, TRUE, TRUE, 0);
	// pack username
	hbox = gtk_hbox_new (FALSE, 3);
	gtk_box_pack_start (GTK_BOX(hbox), view->label_username, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(hbox), view->entry_username, TRUE, TRUE, 0);
	gtk_box_pack_start (GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
	// pack password
	hbox = gtk_hbox_new (FALSE, 3);
	gtk_box_pack_start (GTK_BOX(hbox), view->label_password, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(hbox), view->entry_password, TRUE, TRUE, 0);
	gtk_box_pack_start (GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
	// pack right side
	gtk_box_pack_start (GTK_BOX(view->self), vbox, FALSE, FALSE, 0);


	// change setting view container
	SETTING_VIEW (view)->view_container = view->self;

	on_toggled (view);

	gtk_widget_show_all (view->self);

	g_signal_connect_swapped (view->radio_none, "toggled",
	                          G_CALLBACK(on_toggled), view);
	g_signal_connect_swapped (view->toggle_authentication, "toggled",
	                          G_CALLBACK(on_toggled), view);
}

void proxy_setting_view_set (ProxySettingView* view,
                             ProxySetting* setting)
{
	if (setting==NULL)
		return;

	gtk_entry_set_text (GTK_ENTRY(view->entry_host),
	                    STR_N2E(setting->address));
	gtk_entry_set_text (GTK_ENTRY(view->entry_username),
	                    STR_N2E(setting->username));
	gtk_entry_set_text (GTK_ENTRY(view->entry_password),
	                    STR_N2E(setting->password));
	gtk_spin_button_set_value (GTK_SPIN_BUTTON(view->spin_port),
	                           setting->port);

	switch (setting->type) {
	case PROXY_TYPE_NONE:
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(view->radio_none), TRUE);
		break;
	case PROXY_TYPE_HTTP:
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(view->radio_http), TRUE);
		break;
	case PROXY_TYPE_SOCKS5:
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(view->radio_socks5), TRUE);
	}
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(view->toggle_authentication),
	                              setting->authentication);
}

void proxy_setting_view_get (ProxySettingView* view,
                             ProxySetting* setting)
{
	proxy_setting_set_address (setting,
	                           gtk_entry_get_text (GTK_ENTRY(view->entry_host)));
	proxy_setting_set_username (setting,
	                            gtk_entry_get_text (GTK_ENTRY(view->entry_username)));
	proxy_setting_set_password (setting,
	                            gtk_entry_get_text (GTK_ENTRY(view->entry_password)));
	setting->port = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(view->spin_port));

	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (view->radio_none)))
		setting->type = PROXY_TYPE_NONE;
	else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (view->radio_http)))
		setting->type = PROXY_TYPE_HTTP;
	else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (view->radio_socks5)))
		setting->type = PROXY_TYPE_SOCKS5;

	setting->authentication = gtk_toggle_button_get_active (
	                              GTK_TOGGLE_BUTTON (view->toggle_authentication));
}

void proxy_setting_view_set_multiple (ProxySettingView *view,
                                      gboolean multiple)
{
}

