#include "selection_view.h"
#include "selection_store.h"

static void on_cell_renderer_toggled (GtkCellRendererToggle* cell,
                                      gchar*        path_str,
                                      GtkTreeModel* model)
{
	GtkTreeIter  iter;
	GtkTreePath *path = gtk_tree_path_new_from_string (path_str);
	gboolean fixed;

	gtk_tree_model_get_iter (model, &iter, path);
	gtk_tree_model_get (model, &iter, SELECTION_STORE_SELECTED, &fixed, -1);

	fixed ^= 1;
	gtk_list_store_set (GTK_LIST_STORE(model), &iter,
	                    SELECTION_STORE_SELECTED, fixed, -1);
	gtk_tree_path_free (path);
}


GtkTreeView* selection_view_new (GtkListStore* selection_store,
                                 gchar* column_name)
{
	GtkTreeView*       self;
	GtkCellRenderer*   renderer;
	GtkTreeViewColumn* column;
//	GtkTreeView* treeview;

	self = (GtkTreeView*)gtk_tree_view_new_with_model (
	                         GTK_TREE_MODEL (selection_store));

	gtk_tree_view_set_rules_hint (self, TRUE);
//	gtk_tree_view_set_headers_visible (self, FALSE);

	// column for selected
	renderer = gtk_cell_renderer_toggle_new ();
	g_signal_connect (G_OBJECT(renderer), "toggled",
	                  G_CALLBACK(on_cell_renderer_toggled), selection_store);
	column = gtk_tree_view_column_new_with_attributes ("",
	                                                   renderer,
	                                                   "active",
	                                                   SELECTION_STORE_SELECTED,
	                                                   NULL );
//	gtk_tree_view_column_set_sizing (GTK_TREE_VIEW_COLUMN(column),
//	                                 GTK_TREE_VIEW_COLUMN_FIXED);
//	gtk_tree_view_column_set_fixed_width (GTK_TREE_VIEW_COLUMN(column), 50);
	gtk_tree_view_append_column (self, column);

	// column for string
	renderer = gtk_cell_renderer_text_new ();
	column = gtk_tree_view_column_new_with_attributes (column_name,
	                                                   renderer,
	                                                   "text",
	                                                   SELECTION_STORE_STRING,
	                                                   NULL );
//	gtk_tree_view_column_set_sort_column_id(column, COLUMN_URL);
	gtk_tree_view_append_column (self, column);
	gtk_widget_show_all ((GtkWidget*)self);

	return self;
}

