#include <string.h>
#include <stdlib.h>
#include "urlgfe_parser.h"
#include "global_setting.h"
#include "category_store.h"


void common_parser_end_element (ConfParser*   cp,
                                const gchar*  element_name,
                                gpointer      data,
                                GError**      error)
{
	conf_parser_pop (cp);
}

// proxy setting parser ---------------------------------------------

void proxy_parser_start_element (ConfParser*          cp,
                                 const gchar*         element_name,
                                 const gchar**        attr_names,
                                 const gchar**        attr_values,
                                 ProxySetting*        setting,
                                 GError**             error)
{
	int ii;

	for (ii=0; attr_names[ii]; ii++) {
		if (strcmp (attr_names[ii], "type")==0)
			setting->type = atoi (attr_values[ii]);
		else if (strcmp (attr_names[ii], "authentication")==0)
			setting->authentication = atoi (attr_values[ii]);
		else if (strcmp (attr_names[ii], "port")==0)
			setting->port = atoi (attr_values[ii]);
		else if (strcmp (attr_names[ii], "address")==0)
			setting_set_string (&setting->address, attr_values[ii]);
		else if (strcmp (attr_names[ii], "username")==0)
			setting_set_string (&setting->username, attr_values[ii]);
		else if (strcmp (attr_names[ii], "password")==0)
			setting_set_string (&setting->password, attr_values[ii]);
	}
}

static ConfParserEntry proxy_parser = {
	(conf_parser_start_element_func) proxy_parser_start_element,
	(conf_parser_end_element_func)   common_parser_end_element,
	(conf_parser_text_func)          NULL,
	(conf_parser_passthrough_func)   NULL,
	(conf_parser_error_func)         NULL
};

ConfParserEntry* urlgfe_get_proxy_parser ()
{
	return &proxy_parser;
}

// downloaded setting parser ----------------------------------------

void downloaded_parser_parse_attr (const gchar**        attr_names,
                                   const gchar**        attr_values,
                                   DownloadSetting*     setting)
{
	int ii;

	for (ii=0; attr_names[ii]; ii++) {
		if (strcmp (attr_names[ii], "parts")==0)
			setting->n_parts = atoi (attr_values[ii]);
		if (strcmp (attr_names[ii], "redirect")==0)
			setting->n_redirect = atoi (attr_values[ii]);
		if (strcmp (attr_names[ii], "retries")==0)
			setting->n_retries = atoi (attr_values[ii]);
		if (strcmp (attr_names[ii], "retry_delay")==0)
			setting->retry_delay = atoi (attr_values[ii]);
		if (strcmp (attr_names[ii], "multiple")==0)
			setting->multiple = atoi (attr_values[ii]);
		if (strcmp (attr_names[ii], "start_mode")==0)
			setting->start_mode = atoi (attr_values[ii]);
		if (strcmp (attr_names[ii], "state")==0)
			setting->state = atoi (attr_values[ii]);
	}
}

void downloaded_parser_text (ConfParser*          cp,
                             const gchar*         text,
                             gsize                text_len,
                             gchar**              dest_str,
                             GError**             error)
{
	setting_set_string_len (dest_str, text, text_len);
	conf_parser_pop_data (cp);

	cp->parser->text = NULL;
}

void downloaded_parser_start_element (ConfParser*          cp,
                                      const gchar*         element_name,
                                      const gchar**        attr_names,
                                      const gchar**        attr_values,
                                      DownloadSetting*     setting,
                                      GError**             error)
{
	ConfParserEntry* parser = cp->parser;

	if (strcmp (element_name, PROXY_SETTING_TAG)==0) {
		conf_parser_push (cp, urlgfe_get_proxy_parser (), setting);
		proxy_parser_start_element (cp, element_name, attr_names,
		                            attr_values, PROXY_SETTING(setting), error);
	}
	else if (strcmp (element_name, "url")==0) {
		parser->text = (conf_parser_text_func)downloaded_parser_text;
		conf_parser_push_data (cp, &setting->url);
	}
	else if (strcmp (element_name, "directory")==0) {
		parser->text = (conf_parser_text_func)downloaded_parser_text;
		conf_parser_push_data (cp, &setting->directory);
	}
	else if (strcmp (element_name, "filename")==0) {
		parser->text = (conf_parser_text_func)downloaded_parser_text;
		conf_parser_push_data (cp, &setting->filename);
	}
	else if (strcmp (element_name, "referer")==0) {
		parser->text = (conf_parser_text_func)downloaded_parser_text;
		conf_parser_push_data (cp, &setting->referer);
	}
	else if (strcmp (element_name, DOWNLOAD_SETTING_TAG)==0) {
		downloaded_parser_parse_attr (attr_names, attr_values, setting);
	}
}

void downloaded_parser_end_element (ConfParser*   cp,
                                    const gchar*  element_name,
                                    gpointer      data,
                                    GError**      error)
{
	if (strcmp (element_name, DOWNLOAD_SETTING_TAG)==0)
		conf_parser_pop (cp);
}

static ConfParserEntry downloaded_parser = {
	(conf_parser_start_element_func) downloaded_parser_start_element,
	(conf_parser_end_element_func)   downloaded_parser_end_element,
	(conf_parser_text_func)          NULL,
	(conf_parser_passthrough_func)   NULL,
	(conf_parser_error_func)         NULL
};

ConfParserEntry* urlgfe_get_downloaded_parser ()
{
	return &downloaded_parser;
}

// category setting parser ------------------------------------------

void category_parser_parse_attr (const gchar**        attr_names,
                                 const gchar**        attr_values,
                                 CategorySetting*     setting)
{
	int ii;

	for (ii=0; attr_names[ii]; ii++) {
		if (strcmp (attr_names[ii], "name")==0)
			setting_set_string (&setting->name, attr_values[ii]);
		if (strcmp (attr_names[ii], "state")==0)
			setting->state = atoi (attr_values[ii]);
		if (strcmp (attr_names[ii], "multiple")==0)
			setting->multiple = atoi (attr_values[ii]);
		if (strcmp (attr_names[ii], "n_active_download")==0)
			setting->n_active_download = atoi (attr_values[ii]);

		if (strcmp (attr_names[ii], "completed_capacity")==0)
			setting->completed_capacity = atoi (attr_values[ii]);
		if (strcmp (attr_names[ii], "recycled_capacity")==0)
			setting->recycled_capacity = atoi (attr_values[ii]);

		if (strcmp (attr_names[ii], "col_completed")==0)
			setting->col_completed = atoi (attr_values[ii]);
		if (strcmp (attr_names[ii], "col_total")==0)
			setting->col_total = atoi (attr_values[ii]);
		if (strcmp (attr_names[ii], "col_speed")==0)
			setting->col_speed = atoi (attr_values[ii]);
		if (strcmp (attr_names[ii], "col_retry")==0)
			setting->col_retry = atoi (attr_values[ii]);
		if (strcmp (attr_names[ii], "col_url")==0)
			setting->col_url = atoi (attr_values[ii]);

		if (strcmp (attr_names[ii], "text_filename")==0)
			setting->text_filename = atoi (attr_values[ii]);
		if (strcmp (attr_names[ii], "text_directory")==0)
			setting->text_directory = atoi (attr_values[ii]);
		if (strcmp (attr_names[ii], "text_url")==0)
			setting->text_url = atoi (attr_values[ii]);
		if (strcmp (attr_names[ii], "text_start_mode")==0)
			setting->text_start_mode = atoi (attr_values[ii]);
		if (strcmp (attr_names[ii], "text_message")==0)
			setting->text_message = atoi (attr_values[ii]);
	}
}

void category_parser_start_element (ConfParser*          cp,
                                    const gchar*         element_name,
                                    const gchar**        attr_names,
                                    const gchar**        attr_values,
                                    CategorySetting*     setting,
                                    GError**             error)
{
	if (strcmp (element_name, DOWNLOAD_SETTING_TAG)==0) {
		conf_parser_push (cp, urlgfe_get_downloaded_parser (), setting);
		downloaded_parser_parse_attr (attr_names, attr_values,
		                              DOWNLOAD_SETTING (setting));
	}
	else if (strcmp (element_name, CATEGORY_SETTING_TAG)==0) {
		category_parser_parse_attr (attr_names, attr_values, setting);
	}
}

void category_parser_end_element (ConfParser*   cp,
                                  const gchar*  element_name,
                                  gpointer      data,
                                  GError**      error)
{
	if (strcmp (element_name, CATEGORY_SETTING_TAG)==0)
		conf_parser_pop (cp);
}

static ConfParserEntry category_parser = {
	(conf_parser_start_element_func) category_parser_start_element,
	(conf_parser_end_element_func)   category_parser_end_element,
	(conf_parser_text_func)          NULL,
	(conf_parser_passthrough_func)   NULL,
	(conf_parser_error_func)         NULL
};

ConfParserEntry* urlgfe_get_category_parser ()
{
	return &category_parser;
}

// global setting parser --------------------------------------------

void global_parser_parse_attr (const gchar**        attr_names,
                               const gchar**        attr_values,
                               GlobalSetting*       setting)
{
	int ii;

	for (ii=0; attr_names[ii]; ii++) {
		if (strcmp (attr_names[ii], "n_active_category")==0)
			setting->n_active_category = atoi (attr_values[ii]);
		if (strcmp (attr_names[ii], "header_category_visible")==0)
			setting->header_category_visible = atoi (attr_values[ii]);
		if (strcmp (attr_names[ii], "monitoring_clipboard")==0)
			setting->monitoring_clipboard = atoi (attr_values[ii]);
	}
}

void global_parser_directory_text  (ConfParser*          cp,
                                    const gchar*         text,
                                    gsize                text_len,
                                    GlobalSetting*       setting,
                                    GError**             error)
{
	gchar* str = g_strndup (text, text_len);

	global_setting_add_directory (setting, str);
	g_free (str);

	cp->parser->text = NULL;
}

void global_parser_extention_text  (ConfParser*          cp,
                                    const gchar*         text,
                                    gsize                text_len,
                                    GlobalSetting*       setting,
                                    GError**             error)
{
	gchar* str = g_strndup (text, text_len);

	global_setting_set_extention (setting, str);
	g_free (str);

	cp->parser->text = NULL;
}

void global_parser_start_element (ConfParser*          cp,
                                  const gchar*         element_name,
                                  const gchar**        attr_names,
                                  const gchar**        attr_values,
                                  GlobalSetting*       setting,
                                  GError**             error)
{
	if (strcmp (element_name, CATEGORY_SETTING_TAG)==0) {
		conf_parser_push (cp, urlgfe_get_category_parser (), setting);
		category_parser_parse_attr (attr_names, attr_values, CATEGORY_SETTING (setting));
	}
	else if (strcmp (element_name, "directory")==0) {
		cp->parser->text = (conf_parser_text_func)global_parser_directory_text;
	}
	else if (strcmp (element_name, "extention")==0) {
		cp->parser->text = (conf_parser_text_func)global_parser_extention_text;
	}
	else {
		global_parser_parse_attr (attr_names, attr_values, setting);
	}
}

void global_parser_end_element (ConfParser*   cp,
                                const gchar*  element_name,
                                gpointer      data,
                                GError**      error)
{
	if (strcmp (element_name, GLOBAL_SETTING_TAG)==0)
		conf_parser_pop (cp);
}

static ConfParserEntry global_parser = {
	(conf_parser_start_element_func) global_parser_start_element,
	(conf_parser_end_element_func)   global_parser_end_element,
	(conf_parser_text_func)          NULL,
	(conf_parser_passthrough_func)   NULL,
	(conf_parser_error_func)         NULL
};

ConfParserEntry* urlgfe_get_global_parser ()
{
	return &global_parser;
}

// downloaded store parser ------------------------------------------

void downloaded_store_parser_start_element (ConfParser*          cp,
                                            const gchar*         element_name,
                                            const gchar**        attr_names,
                                            const gchar**        attr_values,
                                            DownloadStore*       store,
                                            GError**             error)
{
	Download* dl_data;

	if (strcmp (element_name, DOWNLOAD_SETTING_TAG)==0)
	{
		dl_data = download_new ();
		download_store_append_allocated (store, dl_data);

		conf_parser_push (cp, urlgfe_get_downloaded_parser (), dl_data);
		downloaded_parser_parse_attr (attr_names, attr_values,
		                              DOWNLOAD_SETTING (dl_data));
	}
}

static ConfParserEntry downloaded_store_parser = {
	(conf_parser_start_element_func) downloaded_store_parser_start_element,
	(conf_parser_end_element_func)   common_parser_end_element,
	(conf_parser_text_func)          NULL,
	(conf_parser_passthrough_func)   NULL,
	(conf_parser_error_func)         NULL
};

// sub category parser ----------------------------------------------

void sub_category_parser_start_element (ConfParser*          cp,
                                        const gchar*         element_name,
                                        const gchar**        attr_names,
                                        const gchar**        attr_values,
                                        Category*            category,
                                        GError**             error)
{
	DownloadStore*   download_store;

	if (strcmp (element_name, CATEGORY_SETTING_TAG)==0) {
		conf_parser_push (cp, urlgfe_get_category_parser (), category);
		category_parser_parse_attr (attr_names, attr_values,
		                            CATEGORY_SETTING (category));
	}
	else {
		if (strcmp (element_name, CATEGORY_WAITING_STORE_TAG)==0 )
			download_store = &category->waiting_store;
		else if (strcmp (element_name, CATEGORY_COMPLETED_STORE_TAG)==0 )
			download_store = &category->completed_store;
		else if (strcmp (element_name, CATEGORY_RECYCLED_STORE_TAG)==0 )
			download_store = &category->recycled_store;

		conf_parser_push (cp, &downloaded_store_parser, download_store);
	}
}

static ConfParserEntry sub_category_parser = {
	(conf_parser_start_element_func) sub_category_parser_start_element,
	(conf_parser_end_element_func)   common_parser_end_element,
	(conf_parser_text_func)          NULL,
	(conf_parser_passthrough_func)   NULL,
	(conf_parser_error_func)         NULL
};

// category store parser --------------------------------------------

void category_store_parser_start_element (ConfParser*          cp,
                                          const gchar*         element_name,
                                          const gchar**        attr_names,
                                          const gchar**        attr_values,
                                          GList**              list,
                                          GError**             error)
{
	Category*        category;

	if (strcmp (element_name, CATEGORY_TAG)==0) {
		category = category_new ();
		*list = g_list_append (*list, category);

		conf_parser_push (cp, &sub_category_parser, category);
	}
}

static ConfParserEntry category_store_parser = {
	(conf_parser_start_element_func) category_store_parser_start_element,
	(conf_parser_end_element_func)   common_parser_end_element,
	(conf_parser_text_func)          NULL,
	(conf_parser_passthrough_func)   NULL,
	(conf_parser_error_func)         NULL
};

// category/setting file reader -------------------------------------

GList* urlgfe_parser_read_category (ConfParser*  cp,
                                    const gchar* filename,
                                    GError** error)
{
	GList* list = NULL;
	gboolean read_ok;

	conf_parser_push (cp, &category_store_parser, &list);
	read_ok = conf_parser_read_file (cp, filename, error);

	return list;
}

GlobalSetting* urlgfe_parser_read_setting (ConfParser*  cp,
                                           const gchar* filename,
                                           GError** error)
{
	GlobalSetting* setting = global_setting_new ();

	conf_parser_push (cp, urlgfe_get_global_parser (), setting);
	conf_parser_read_file (cp, filename, error);

	return setting;
}

